@@ -1,19 +1,20 @@
-/* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 1999, 2001, 2005 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "imap4d.h"
 
@@ -24,13 +25,15 @@ imap4d_append (struct imap4d_command *command, char *arg)
   char *sp;
   char *mboxname;
   int flags = 0;
-  mailbox_t dest_mbox = NULL;
+  mu_mailbox_t dest_mbox = NULL;
   int status;
 
   mboxname = util_getword (arg, &sp);
   if (!mboxname)
     return util_finish (command, RESP_BAD, "Too few arguments");
 
+  util_unquote (&mboxname);
+  
   if (*sp == '(' && util_parse_attributes (sp+1, &sp, &flags))
     return util_finish (command, RESP_BAD, "Missing closing parenthesis");
   
@@ -38,17 +41,17 @@ imap4d_append (struct imap4d_command *command, char *arg)
   if (!mboxname)
     return util_finish (command, RESP_NO, "Couldn't open mailbox"); 
 
-  status = mailbox_create_default (&dest_mbox, mboxname);
+  status = mu_mailbox_create_default (&dest_mbox, mboxname);
   if (status == 0)
     {
       /* It SHOULD NOT automatifcllly create the mailbox. */
-      status = mailbox_open (dest_mbox, MU_STREAM_RDWR);
+      status = mu_mailbox_open (dest_mbox, MU_STREAM_RDWR);
       if (status == 0)
 	{
 	  status = imap4d_append0 (dest_mbox, flags, sp);
-	  mailbox_close (dest_mbox);
+	  mu_mailbox_close (dest_mbox);
 	}
-      mailbox_destroy (&dest_mbox);
+      mu_mailbox_destroy (&dest_mbox);
     }
   
   free (mboxname);
@@ -59,40 +62,40 @@ imap4d_append (struct imap4d_command *command, char *arg)
 }
 
 static int
-_append_date (envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
+_append_date (mu_envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
 {
-  message_t msg = envelope_get_owner (envelope);
-  struct tm **tm = message_get_owner (msg);
+  mu_message_t msg = mu_envelope_get_owner (envelope);
+  struct tm **tm = mu_message_get_owner (msg);
 
   strftime (buf, len, "%a %b %d %H:%M:%S %Y", *tm);
   return 0;
 }
 
 static int
-_append_sender (envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
+_append_sender (mu_envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
 {
   strncpy (buf, "GNU-imap4d", len);
   return 0;
 }
 
 int
-imap4d_append0 (mailbox_t mbox, int flags, char *text)
+imap4d_append0 (mu_mailbox_t mbox, int flags, char *text)
 {
-  stream_t stream;
+  mu_stream_t stream;
   int rc = 0;
   size_t len = 0;
-  message_t msg = 0;
+  mu_message_t msg = 0;
   struct tm *tm;
   time_t t;
-  envelope_t env;
+  mu_envelope_t env;
     
-  if (message_create (&msg, &tm))
+  if (mu_message_create (&msg, &tm))
     return 1;
   
-  if (memory_stream_create (&stream, 0, MU_STREAM_RDWR)
-      || stream_open (stream))
+  if (mu_memory_stream_create (&stream, 0, MU_STREAM_RDWR)
+      || mu_stream_open (stream))
     {
-      message_destroy (&msg, &tm);
+      mu_message_destroy (&msg, &tm);
       return 1;
     }
 
@@ -113,25 +116,25 @@ imap4d_append0 (mailbox_t mbox, int flags, char *text)
     }
   tm = gmtime(&t);
 
-  stream_write (stream, text, strlen (text), len, &len);
-  message_set_stream (msg, stream, &tm);
+  mu_stream_write (stream, text, strlen (text), len, &len);
+  mu_message_set_stream (msg, stream, &tm);
 
-  envelope_create (&env, msg);
-  envelope_set_date (env, _append_date, msg);
-  envelope_set_sender (env, _append_sender, msg);
-  message_set_envelope (msg, env, &tm);
-  rc = mailbox_append_message (mbox, msg);
+  mu_envelope_create (&env, msg);
+  mu_envelope_set_date (env, _append_date, msg);
+  mu_envelope_set_sender (env, _append_sender, msg);
+  mu_message_set_envelope (msg, env, &tm);
+  rc = mu_mailbox_append_message (mbox, msg);
   if (rc == 0 && flags)
     {
       size_t num = 0;
-      attribute_t attr = NULL;
-      mailbox_messages_count (mbox, &num);
-      mailbox_get_message (mbox, num, &msg);
-      message_get_attribute (msg, &attr);
-      attribute_set_flags (attr, flags);
+      mu_attribute_t attr = NULL;
+      mu_mailbox_messages_count (mbox, &num);
+      mu_mailbox_get_message (mbox, num, &msg);
+      mu_message_get_attribute (msg, &attr);
+      mu_attribute_set_flags (attr, flags);
     }
 
-  message_destroy (&msg, &tm);
+  mu_message_destroy (&msg, &tm);
   return rc;
 }
 
