@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,41 +36,41 @@
 
 struct fetch_command;
 
-static int fetch_all               __P ((struct fetch_command *, char**));
-static int fetch_full              __P ((struct fetch_command *, char**));
-static int fetch_fast              __P ((struct fetch_command *, char**));
-static int fetch_envelope          __P ((struct fetch_command *, char**));
-static int fetch_flags             __P ((struct fetch_command *, char**));
-static int fetch_internaldate      __P ((struct fetch_command *, char**));
-static int fetch_rfc822_header     __P ((struct fetch_command *, char**));
-static int fetch_rfc822_size       __P ((struct fetch_command *, char**));
-static int fetch_rfc822_text       __P ((struct fetch_command *, char**));
-static int fetch_rfc822            __P ((struct fetch_command *, char**));
-static int fetch_bodystructure     __P ((struct fetch_command *, char**));
-static int fetch_body              __P ((struct fetch_command *, char**));
-static int fetch_uid               __P ((struct fetch_command *, char**));
+static int fetch_all               (struct fetch_command *, char**);
+static int fetch_full              (struct fetch_command *, char**);
+static int fetch_fast              (struct fetch_command *, char**);
+static int fetch_envelope          (struct fetch_command *, char**);
+static int fetch_flags             (struct fetch_command *, char**);
+static int fetch_internaldate      (struct fetch_command *, char**);
+static int fetch_rfc822_header     (struct fetch_command *, char**);
+static int fetch_rfc822_size       (struct fetch_command *, char**);
+static int fetch_rfc822_text       (struct fetch_command *, char**);
+static int fetch_rfc822            (struct fetch_command *, char**);
+static int fetch_bodystructure     (struct fetch_command *, char**);
+static int fetch_body              (struct fetch_command *, char**);
+static int fetch_uid               (struct fetch_command *, char**);
 
 /* Helper functions.  */
-static int fetch_envelope0         __P ((message_t));
-static int fetch_bodystructure0    __P ((message_t, int));
-static int bodystructure           __P ((message_t, int));
-static void send_parameter_list    __P ((const char *));
-static int fetch_operation         __P ((message_t, char **, int));
-static int fetch_message           __P ((message_t, unsigned long, unsigned long));
-static int fetch_header            __P ((message_t, unsigned long, unsigned long));
-static int fetch_body_content      __P ((message_t, unsigned long, unsigned long));
-static int fetch_io                __P ((stream_t, unsigned long, unsigned long, unsigned long));
-static int fetch_header_fields     __P ((message_t, char **, unsigned long, unsigned long));
-static int fetch_header_fields_not __P ((message_t, char **, unsigned long, unsigned long));
-static int fetch_send_address      __P ((const char *));
+static int fetch_envelope0         (mu_message_t);
+static int fetch_bodystructure0    (mu_message_t, int);
+static int bodystructure           (mu_message_t, int);
+static void send_parameter_list    (const char *);
+static int fetch_operation         (mu_message_t, char **, int);
+static int fetch_message           (mu_message_t, unsigned long, unsigned long);
+static int fetch_header            (mu_message_t, unsigned long, unsigned long);
+static int fetch_body_content      (mu_message_t, unsigned long, unsigned long);
+static int fetch_io                (mu_stream_t, unsigned long, unsigned long, unsigned long);
+static int fetch_header_fields     (mu_message_t, char **, unsigned long, unsigned long);
+static int fetch_header_fields_not (mu_message_t, char **, unsigned long, unsigned long);
+static int fetch_send_address      (const char *);
 
-static struct fetch_command* fetch_getcommand __P ((char *, struct fetch_command[]));
+static struct fetch_command* fetch_getcommand (char *, struct fetch_command*);
 
 struct fetch_command
 {
   const char *name;
-  int (*func) __P ((struct fetch_command *, char **));
-  message_t msg;
+  int (*func) (struct fetch_command *, char **);
+  mu_message_t msg;
 } fetch_command_table [] =
 {
 #define F_ALL 0
@@ -105,7 +105,7 @@ struct fetch_command
 /* Go through the fetch array sub command and returns the the structure.  */
 
 static struct fetch_command *
-fetch_getcommand (char *cmd, struct fetch_command command_table[])
+fetch_getcommand (char *cmd, struct fetch_command *command_table)
 {
   size_t i, len = strlen (cmd);
 
@@ -128,7 +128,7 @@ imap4d_fetch (struct imap4d_command *command, char *arg)
   char buffer[64];
 
   rc = imap4d_fetch0 (arg, 0, buffer, sizeof buffer);
-  return util_finish (command, rc, buffer);
+  return util_finish (command, rc, "%s", buffer);
 }
 
 /* Where the real implementation is.  It is here since UID command also
@@ -167,9 +167,9 @@ imap4d_fetch0 (char *arg, int isuid, char *resp, size_t resplen)
   for (i = 0; i < n && rc == RESP_OK; i++)
     {
       size_t msgno = (isuid) ? uid_to_msgno (set[i]) : set[i];
-      message_t msg = NULL;
+      mu_message_t msg = NULL;
 
-      if (msgno && mailbox_get_message (mbox, msgno, &msg) == 0)
+      if (msgno && mu_mailbox_get_message (mbox, msgno, &msg) == 0)
 	{
 	  char item[32];
 	  char *items = strdup (sp);
@@ -177,7 +177,7 @@ imap4d_fetch0 (char *arg, int isuid, char *resp, size_t resplen)
 	  int space = 0;
 
 	  fcmd = NULL;
-	  util_send ("* %d FETCH (", msgno);
+	  util_send ("* %s FETCH (", mu_umaxtostr (0, msgno));
 	  item[0] = '\0';
 	  /* Server implementations MUST implicitly
 	     include the UID message data item as part of any FETCH
@@ -214,7 +214,9 @@ imap4d_fetch0 (char *arg, int isuid, char *resp, size_t resplen)
 	  util_send (")\r\n");
 	  free (p);
 	}
-      else
+      else if (!isuid)
+	/* According to RFC 3501, "A non-existent unique identifier is
+	   ignored without any error message generated." */
 	{
 	  snprintf (resp, resplen,
 		    "Bogus message set: message number out of range");
@@ -290,11 +292,11 @@ fetch_envelope (struct fetch_command *command, char **arg ARG_UNUSED)
 /* FLAGS: The flags that are set for this message.  */
 /* FIXME: User flags not done. If enable change the PERMANENTFLAGS in SELECT */
 void
-fetch_flags0 (const char *prefix, message_t msg, int isuid)
+fetch_flags0 (const char *prefix, mu_message_t msg, int isuid)
 {
-  attribute_t attr = NULL;
+  mu_attribute_t attr = NULL;
 
-  message_get_attribute (msg, &attr);
+  mu_message_get_attribute (msg, &attr);
   if (isuid)
     {
       struct fetch_command *fcmd = &fetch_command_table[F_UID];
@@ -348,13 +350,13 @@ static int
 fetch_internaldate (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   char date[128];
-  envelope_t env = NULL;
+  mu_envelope_t env = NULL;
   struct tm tm, *tmp = NULL;
   mu_timezone tz;
 
-  message_get_envelope (command->msg, &env);
+  mu_message_get_envelope (command->msg, &env);
   date[0] = '\0';
-  if (envelope_date (env, date, sizeof (date), NULL) == 0)
+  if (mu_envelope_date (env, date, sizeof (date), NULL) == 0)
     {
       char *p = date;
       if (mu_parse_ctime_date_time ((const char **) &p, &tm, &tz) == 0)
@@ -405,8 +407,8 @@ fetch_rfc822_size (struct fetch_command *command, char **arg ARG_UNUSED)
   size_t size = 0;
   size_t lines = 0;
   
-  message_size (command->msg, &size);
-  message_lines (command->msg, &lines);
+  mu_message_size (command->msg, &size);
+  mu_message_lines (command->msg, &lines);
   util_send ("%s %u", command->name, size + lines);
   return RESP_OK;
 }
@@ -459,8 +461,8 @@ fetch_uid (struct fetch_command *command, char **arg ARG_UNUSED)
 {
   size_t uid = 0;
 
-  message_get_uid (command->msg, &uid);
-  util_send ("%s %d", command->name, uid);
+  mu_message_get_uid (command->msg, &uid);
+  util_send ("%s %s", command->name, mu_umaxtostr (0, uid));
   return RESP_OK;
 }
 
@@ -494,12 +496,12 @@ fetch_body (struct fetch_command *command, char **arg)
   /* It's body section, set the message as seen  */
   if (**arg == '[')
     {
-      attribute_t attr = NULL;
-      message_get_attribute (command->msg, &attr);
-      if (!attribute_is_read (attr))
+      mu_attribute_t attr = NULL;
+      mu_message_get_attribute (command->msg, &attr);
+      if (!mu_attribute_is_read (attr))
 	{
 	  util_send ("FLAGS (\\Seen) ");
-	  attribute_set_read (attr);
+	  mu_attribute_set_read (attr);
 	}
     }
   else if (strncasecmp (*arg,".PEEK", 5) == 0)
@@ -525,14 +527,14 @@ fetch_body (struct fetch_command *command, char **arg)
 /* Helper Functions: Where the Beef is.  */
 
 static void
-fetch_send_header_value (header_t header, const char *name,
+fetch_send_header_value (mu_header_t header, const char *name,
 			 const char *defval, int space)
 {
   char *buffer;
   
   if (space)
     util_send (" ");
-  if (header_aget_value (header, name, &buffer) == 0)
+  if (mu_header_aget_value (header, name, &buffer) == 0)
     {
       util_send_qstring (buffer);
       free (buffer);
@@ -544,14 +546,14 @@ fetch_send_header_value (header_t header, const char *name,
 }
 
 static void
-fetch_send_header_list (header_t header, const char *name,
+fetch_send_header_list (mu_header_t header, const char *name,
 			const char *defval, int space)
 {
   char *buffer;
   
   if (space)
     util_send (" ");
-  if (header_aget_value (header, name, &buffer) == 0)
+  if (mu_header_aget_value (header, name, &buffer) == 0)
     {
       send_parameter_list (buffer);
       free (buffer);
@@ -563,14 +565,14 @@ fetch_send_header_list (header_t header, const char *name,
 }
 
 static void
-fetch_send_header_address (header_t header, const char *name,
+fetch_send_header_address (mu_header_t header, const char *name,
 			   const char *defval, int space)
 {
   char *buffer;
   
   if (space)
     util_send (" ");
-  if (header_aget_value (header, name, &buffer) == 0)
+  if (mu_header_aget_value (header, name, &buffer) == 0)
     {
       fetch_send_address (buffer);
       free (buffer);
@@ -590,18 +592,18 @@ fetch_send_header_address (header_t header, const char *name,
    fields are strings.  The from, sender, reply-to, to, cc, and bcc fields
    are parenthesized lists of address structures.  */
 static int
-fetch_envelope0 (message_t msg)
+fetch_envelope0 (mu_message_t msg)
 {
   char *from = NULL;
-  header_t header = NULL;
+  mu_header_t header = NULL;
 
-  message_get_header (msg, &header);
+  mu_message_get_header (msg, &header);
 
   fetch_send_header_value (header, "Date", NULL, 0);
   fetch_send_header_value (header, "Subject", NULL, 1);
 
   /* From:  */
-  header_aget_value (header, "From", &from);
+  mu_header_aget_value (header, "From", &from);
   util_send (" ");
   fetch_send_address (from);
 
@@ -640,41 +642,41 @@ fetch_envelope0 (message_t msg)
    A string or parenthesized list giving the body language value as defined
    in [LANGUAGE-TAGS].  */
 static int
-fetch_bodystructure0 (message_t message, int extension)
+fetch_bodystructure0 (mu_message_t message, int extension)
 {
   size_t nparts = 1;
   size_t i;
   int is_multipart = 0;
 
-  message_is_multipart (message, &is_multipart);
+  mu_message_is_multipart (message, &is_multipart);
   if (is_multipart)
     {
       char *buffer = NULL;
-      header_t header = NULL;
+      mu_header_t header = NULL;
 
-      message_get_num_parts (message, &nparts);
+      mu_message_get_num_parts (message, &nparts);
 
       /* Get all the sub messages.  */
       for (i = 1; i <= nparts; i++)
         {
-          message_t msg = NULL;
-          message_get_part (message, i, &msg);
+          mu_message_t msg = NULL;
+          mu_message_get_part (message, i, &msg);
           util_send ("(");
           fetch_bodystructure0 (msg, extension);
           util_send (")");
         } /* for () */
 
-      message_get_header (message, &header);
+      mu_message_get_header (message, &header);
 
 
       /* The subtype.  */
-      if (header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
+      if (mu_header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
 	{
 	  int argc = 0;
 	  char **argv;
 	  char *s;
 	  
-	  argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
+	  mu_argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
 
 	  s = strchr (argv[0], '/');
 	  if (s)
@@ -731,7 +733,7 @@ fetch_bodystructure0 (message_t message, int extension)
 	    }
 	  else
 	    util_send (" NIL");
-	  argcv_free (argc, argv);
+	  mu_argcv_free (argc, argv);
           free (buffer);
 	}
       else
@@ -795,23 +797,23 @@ fetch_bodystructure0 (message_t message, int extension)
    in [LANGUAGE-TAGS].
  */
 static int
-bodystructure (message_t msg, int extension)
+bodystructure (mu_message_t msg, int extension)
 {
-  header_t header = NULL;
+  mu_header_t header = NULL;
   char *buffer = NULL;
   size_t blines = 0;
   int message_rfc822 = 0;
   int text_plain = 0;
 
-  message_get_header (msg, &header);
+  mu_message_get_header (msg, &header);
 
-  if (header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
+  if (mu_header_aget_value (header, MU_HEADER_CONTENT_TYPE, &buffer) == 0)
     {
       int argc = 0;
       char **argv;
       char *s, *p;
 	  
-      argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
+      mu_argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
 
       if (strcasecmp (argv[0], "MESSAGE/RFC822") == 0)
         message_rfc822 = 1;
@@ -888,7 +890,7 @@ bodystructure (message_t msg, int extension)
 	}
       else
 	util_send (" NIL");
-      argcv_free (argc, argv);
+      mu_argcv_free (argc, argv);
       free (buffer);
     }
   else
@@ -910,24 +912,24 @@ bodystructure (message_t msg, int extension)
   /* body size RFC822 format.  */
   {
     size_t size = 0;
-    body_t body = NULL;
-    message_get_body (msg, &body);
-    body_size (body, &size);
-    body_lines (body, &blines);
-    util_send (" %d", size + blines);
+    mu_body_t body = NULL;
+    mu_message_get_body (msg, &body);
+    mu_body_size (body, &size);
+    mu_body_lines (body, &blines);
+    util_send (" %s", mu_umaxtostr (0, size + blines));
   }
 
   /* If the mime type was text.  */
   if (text_plain)
     {
       /* Add the line number of the body.  */
-      util_send (" %d", blines);
+      util_send (" %s", mu_umaxtostr (0, blines));
     }
   else if (message_rfc822)
     {
       size_t lines = 0;
-      message_t emsg = NULL;
-      message_unencapsulate  (msg, &emsg, NULL);
+      mu_message_t emsg = NULL;
+      mu_message_unencapsulate  (msg, &emsg, NULL);
       /* Add envelope structure of the encapsulated message.  */
       util_send (" (");
       fetch_envelope0 (emsg);
@@ -937,9 +939,9 @@ bodystructure (message_t msg, int extension)
       bodystructure (emsg, extension);
       util_send (")");
       /* Size in text lines of the encapsulated message.  */
-      message_lines (emsg, &lines);
-      util_send (" %d", lines);
-      message_destroy (&emsg, NULL);
+      mu_message_lines (emsg, &lines);
+      util_send (" %s", mu_umaxtostr (0, lines));
+      mu_message_destroy (&emsg, NULL);
     }
 
   if (extension)
@@ -957,7 +959,7 @@ bodystructure (message_t msg, int extension)
 }
 
 static int
-fetch_operation (message_t msg, char **arg, int silent)
+fetch_operation (mu_message_t msg, char **arg, int silent)
 {
   unsigned long start = ULONG_MAX; /* No starting offset.  */
   unsigned long end = ULONG_MAX; /* No limit. */
@@ -995,7 +997,7 @@ fetch_operation (message_t msg, char **arg, int silent)
 	break;
 
       /* If the section message did not exist bail out here.  */
-      status = message_get_part (msg, j, &msg);
+      status = mu_message_get_part (msg, j, &msg);
       if (status != 0)
 	{
 	  util_send (" \"\"");
@@ -1106,51 +1108,51 @@ fetch_operation (message_t msg, char **arg, int silent)
 }
 
 static int
-fetch_message (message_t msg, unsigned long start, unsigned long end)
+fetch_message (mu_message_t msg, unsigned long start, unsigned long end)
 {
-  stream_t stream = NULL;
+  mu_stream_t stream = NULL;
   size_t size = 0, lines = 0;
-  message_get_stream (msg, &stream);
-  message_size (msg, &size);
-  message_lines (msg, &lines);
+  mu_message_get_stream (msg, &stream);
+  mu_message_size (msg, &size);
+  mu_message_lines (msg, &lines);
   return fetch_io (stream, start, end, size + lines);
 }
 
 static int
-fetch_header (message_t msg, unsigned long start, unsigned long end)
+fetch_header (mu_message_t msg, unsigned long start, unsigned long end)
 {
-  header_t header = NULL;
-  stream_t stream = NULL;
+  mu_header_t header = NULL;
+  mu_stream_t stream = NULL;
   size_t size = 0, lines = 0;
-  message_get_header (msg, &header);
-  header_size (header, &size);
-  header_lines (header, &lines);
-  header_get_stream (header, &stream);
+  mu_message_get_header (msg, &header);
+  mu_header_size (header, &size);
+  mu_header_lines (header, &lines);
+  mu_header_get_stream (header, &stream);
   return fetch_io (stream, start, end, size + lines);
 }
 
 static int
-fetch_body_content (message_t msg, unsigned long start, unsigned long end)
+fetch_body_content (mu_message_t msg, unsigned long start, unsigned long end)
 {
-  body_t body = NULL;
-  stream_t stream = NULL;
+  mu_body_t body = NULL;
+  mu_stream_t stream = NULL;
   size_t size = 0, lines = 0;
-  message_get_body (msg, &body);
-  body_size (body, &size);
-  body_lines (body, &lines);
-  body_get_stream (body, &stream);
+  mu_message_get_body (msg, &body);
+  mu_body_size (body, &size);
+  mu_body_lines (body, &lines);
+  mu_body_get_stream (body, &stream);
   return fetch_io (stream, start, end, size + lines);
 }
 
 static int
-fetch_io (stream_t stream, unsigned long start, unsigned long end,
+fetch_io (mu_stream_t stream, unsigned long start, unsigned long end,
 	  unsigned long max)
 {
-  stream_t rfc = NULL;
+  mu_stream_t rfc = NULL;
   size_t n = 0;
-  off_t offset;
+  mu_off_t offset;
 
-  filter_create (&rfc, stream, "rfc822", MU_FILTER_ENCODE, MU_STREAM_READ);
+  mu_filter_create (&rfc, stream, "rfc822", MU_FILTER_ENCODE, MU_STREAM_READ);
 
   if (start == ULONG_MAX || end == ULONG_MAX)
     {
@@ -1159,7 +1161,7 @@ fetch_io (stream_t stream, unsigned long start, unsigned long end,
       if (max)
 	{
 	  util_send (" {%u}\r\n", max);
-	  while (stream_read (rfc, buffer, sizeof (buffer) - 1, offset,
+	  while (mu_stream_read (rfc, buffer, sizeof (buffer) - 1, offset,
 			      &n) == 0 && n > 0)
 	    {
 	      buffer[n] = '\0';
@@ -1181,7 +1183,7 @@ fetch_io (stream_t stream, unsigned long start, unsigned long end,
       offset = start;
       p = buffer = calloc (end + 2, 1);
       while (end > 0
-	     && stream_read (rfc, buffer, end + 1, offset, &n) == 0 && n > 0)
+	     && mu_stream_read (rfc, buffer, end + 1, offset, &n) == 0 && n > 0)
 	{
 	  offset += n;
 	  total += n;
@@ -1193,7 +1195,7 @@ fetch_io (stream_t stream, unsigned long start, unsigned long end,
       util_send ("<%lu>", start);
       if (total)
 	{
-	  util_send (" {%u}\r\n", total);
+	  util_send (" {%s}\r\n", mu_umaxtostr (0, total));
 	  util_send ("%s", p);
 	}
       else
@@ -1204,7 +1206,7 @@ fetch_io (stream_t stream, unsigned long start, unsigned long end,
 }
 
 static int
-fetch_header_fields (message_t msg, char **arg, unsigned long start,
+fetch_header_fields (mu_message_t msg, char **arg, unsigned long start,
 		     unsigned long end)
 {
   char *buffer = NULL;
@@ -1212,10 +1214,10 @@ fetch_header_fields (message_t msg, char **arg, unsigned long start,
   size_t array_len = 0;
   size_t off = 0;
   size_t lines = 0;
-  stream_t stream = NULL;
+  mu_stream_t stream = NULL;
   int status;
 
-  status = memory_stream_create (&stream, 0, 0);
+  status = mu_memory_stream_create (&stream, 0, 0);
   if (status != 0)
     imap4d_bye (ERR_NO_MEM);
 
@@ -1246,17 +1248,17 @@ fetch_header_fields (message_t msg, char **arg, unsigned long start,
   /* Get the header values.  */
   {
     size_t j;
-    header_t header = NULL;
-    message_get_header (msg, &header);
+    mu_header_t header = NULL;
+    mu_message_get_header (msg, &header);
     for (j = 0; j < array_len; j++)
       {
 	char *value = NULL;
 	size_t n = 0;
-	if (header_aget_value (header, array[j], &value))
+	if (mu_header_aget_value (header, array[j], &value))
 	    continue;
 
 	n = asprintf (&buffer, "%s: %s\n", array[j], value);
-	status = stream_write (stream, buffer, n, off, &n);
+	status = mu_stream_write (stream, buffer, n, off, &n);
 	off += n;
 	/* count the lines.  */
 	{
@@ -1275,7 +1277,7 @@ fetch_header_fields (message_t msg, char **arg, unsigned long start,
       }
   }
   /* Headers are always sent with the NL separator.  */
-  stream_write (stream, "\n", 1, off, NULL);
+  mu_stream_write (stream, "\n", 1, off, NULL);
   off++;
   lines++;
 
@@ -1302,7 +1304,7 @@ fetch_header_fields (message_t msg, char **arg, unsigned long start,
 }
 
 static int
-fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
+fetch_header_fields_not (mu_message_t msg, char **arg, unsigned long start,
 			 unsigned long end)
 {
   char **array = NULL;
@@ -1310,10 +1312,10 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
   char *buffer = NULL;
   size_t off = 0;
   size_t lines = 0;
-  stream_t stream = NULL;
+  mu_stream_t stream = NULL;
   int status;
 
-  status = memory_stream_create (&stream, 0, 0);
+  status = mu_memory_stream_create (&stream, 0, 0);
   if (status)
     imap4d_bye (ERR_NO_MEM);
 
@@ -1344,10 +1346,10 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
   /* Build the memory buffer.  */
   {
     size_t i;
-    header_t header = NULL;
+    mu_header_t header = NULL;
     size_t count = 0;
-    message_get_header (msg, &header);
-    header_get_field_count (header, &count);
+    mu_message_get_header (msg, &header);
+    mu_header_get_field_count (header, &count);
     for (i = 1; i <= count; i++)
       {
 	char *name = NULL;
@@ -1356,7 +1358,7 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
 	size_t ignore = 0;
 
 	/* Get the field name.  */
-	status = header_aget_field_name (header, i, &name);
+	status = mu_header_aget_field_name (header, i, &name);
 	if (*name == '\0')
 	  {
 	    free (name);
@@ -1381,13 +1383,13 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
 	    }
 	}
 
-	if (header_aget_field_value (header, i, &value) == 0)
+	if (mu_header_aget_field_value (header, i, &value) == 0)
 	  {
 	    char *nl;
 	    
 	    /* Save the field.  */
 	    n = asprintf (&buffer, "%s: %s\n", name, value);
-	    status = stream_write (stream, buffer, n, off, &n);
+	    status = mu_stream_write (stream, buffer, n, off, &n);
 	    off += n;
 	    /* count the lines.  */
 	    for (nl = buffer;(nl = strchr (nl, '\n')); nl++)
@@ -1406,7 +1408,7 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
       }
   }
   /* Headers are always sent with a NL separator.  */
-  stream_write (stream, "\n", 1, off, NULL);
+  mu_stream_write (stream, "\n", 1, off, NULL);
   off++;
   lines++;
 
@@ -1436,7 +1438,7 @@ fetch_header_fields_not (message_t msg, char **arg, unsigned long start,
 static int
 fetch_send_address (const char *addr)
 {
-  address_t address;
+  mu_address_t address;
   size_t i, count = 0;
 
   /* Short circuit.  */
@@ -1446,8 +1448,8 @@ fetch_send_address (const char *addr)
       return RESP_OK;
     }
 
-  address_create (&address, addr);
-  address_get_count (address, &count);
+  mu_address_create (&address, addr);
+  mu_address_get_count (address, &count);
 
   /* We failed: can't parse.  */
   if (count == 0)
@@ -1464,13 +1466,13 @@ fetch_send_address (const char *addr)
       util_send ("(");
 
       *buf = '\0';
-      address_get_personal (address, i, buf, sizeof (buf), NULL);
+      mu_address_get_personal (address, i, buf, sizeof (buf), NULL);
       util_send_qstring (buf);
 
       util_send (" ");
 
       *buf = '\0';
-      address_get_route (address, i, buf, sizeof (buf), NULL);
+      mu_address_get_route (address, i, buf, sizeof (buf), NULL);
       util_send_qstring (buf);
 
       util_send (" ");
@@ -1479,18 +1481,18 @@ fetch_send_address (const char *addr)
       {
 	int is_group = 0;
 
-	address_is_group (address, i, &is_group);
+	mu_address_is_group (address, i, &is_group);
 	if (is_group)
-	  address_get_personal (address, i, buf, sizeof (buf), NULL);
+	  mu_address_get_personal (address, i, buf, sizeof (buf), NULL);
 	else
-	  address_get_local_part (address, i, buf, sizeof (buf), NULL);
+	  mu_address_get_local_part (address, i, buf, sizeof (buf), NULL);
       }
       util_send_qstring (buf);
 
       util_send (" ");
 
       *buf = '\0';
-      address_get_domain (address, i, buf, sizeof (buf), NULL);
+      mu_address_get_domain (address, i, buf, sizeof (buf), NULL);
       util_send_qstring (buf);
 
       util_send (")");
@@ -1512,7 +1514,7 @@ send_parameter_list (const char *buffer)
       return;
     }
 
-  argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
+  mu_argcv_get (buffer, " \t\r\n;=", NULL, &argc, &argv);
   
   if (argc == 0)
     util_send ("NIL");
@@ -1573,7 +1575,7 @@ send_parameter_list (const char *buffer)
 	util_send (" NIL");
       util_send (")");
     }
-  argcv_free (argc, argv);
+  mu_argcv_free (argc, argv);
 }
 	  
 
