@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
+   2005, 2006 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,15 +14,16 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "imap4d.h"
 
-static stream_t istream;
-static stream_t ostream;
+static mu_stream_t istream;
+static mu_stream_t ostream;
 
-static int add2set __P ((size_t **, int *, unsigned long));
-static const char *sc2string __P ((int));
+static int add2set (size_t **, int *, unsigned long);
+static const char *sc2string (int);
 
 /* Get the next space/CR/NL separated word, some words are between double
    quotes, strtok() can not handle it.  */
@@ -167,16 +169,18 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
   size_t max = 0;
   size_t *tmp;
   int i, j;
-
-  status = mailbox_messages_count (mbox, &max);
+  unsigned long invalid_uid = 0; /* For UID mode only: have we 
+				    encountered an uid > max uid? */
+  
+  status = mu_mailbox_messages_count (mbox, &max);
   if (status != 0)
     return status;
   /* If it is a uid sequence, override max with the UID.  */
   if (isuid)
     {
-      message_t msg = NULL;
-      mailbox_get_message (mbox, max, &msg);
-      message_get_uid (msg, &max);
+      mu_message_t msg = NULL;
+      mu_mailbox_get_message (mbox, max, &msg);
+      mu_message_get_uid (msg, &max);
     }
 
   *n = 0;
@@ -206,6 +210,19 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 		*n = 0;
 		return EINVAL;
 	      }
+	    else if (val > max)
+	      {
+		if (isuid)
+		  {
+		    invalid_uid = 1;
+		    continue;
+		  }
+		if (*set)
+		  free (*set);
+		*n = 0;
+		return EINVAL;
+	      }
+	    
 	    if (low)
 	      {
 		/* Reverse it. */
@@ -284,6 +301,16 @@ util_msgset (char *s, size_t ** set, int *n, int isuid)
 	break;
     }				/* while */
 
+  /* For message sets in form X:Y where Y is a not-existing UID greater
+     than max UID, replace Y with the max UID in the mailbox */
+  if (*n == 1 && invalid_uid)
+    {
+      val = max;
+      status = add2set (set, n, val);
+      if (status != 0)
+	return status;
+    }
+  
   if (low)
     {
       /* Reverse it. */
@@ -336,7 +363,7 @@ util_send (const char *format, ...)
   if (daemon_param.transcript)
     syslog (LOG_DEBUG, "sent: %s", buf);
 
-  status = stream_sequential_write (ostream, buf, strlen (buf));
+  status = mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
 
   return status;
@@ -390,7 +417,7 @@ util_out (int rc, const char *format, ...)
   if (daemon_param.transcript)
     syslog (LOG_DEBUG, "sent: %s", buf);
 
-  status = stream_sequential_write (ostream, buf, strlen (buf));
+  status = mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
   free (tempbuf);
   return status;
@@ -400,26 +427,40 @@ util_out (int rc, const char *format, ...)
 int
 util_finish (struct imap4d_command *command, int rc, const char *format, ...)
 {
-  char *tempbuf = NULL;
+  size_t size;
   char *buf = NULL;
+  char *tempbuf = NULL;
   int new_state;
   int status = 0;
   va_list ap;
-
-  asprintf (&tempbuf, "%s %s%s %s\r\n", command->tag, sc2string (rc),
-	    command->name, format);
+  const char *sc = sc2string (rc);
+  
   va_start (ap, format);
-  vasprintf (&buf, tempbuf, ap);
+  vasprintf (&tempbuf, format, ap);
   va_end (ap);
+  if (!tempbuf)
+    imap4d_bye (ERR_NO_MEM);
+  
+  size = strlen (command->tag) + 1 +
+         strlen (sc) + strlen (command->name) + 1 +
+         strlen (tempbuf) + 1;
+  buf = malloc (size);
   if (!buf)
     imap4d_bye (ERR_NO_MEM);
+  strcpy (buf, command->tag);
+  strcat (buf, " ");
+  strcat (buf, sc);
+  strcat (buf, command->name);
+  strcat (buf, " ");
+  strcat (buf, tempbuf);
+  free (tempbuf);
 
   if (daemon_param.transcript)
-    syslog (LOG_DEBUG, "sent: %s", buf);
+    syslog (LOG_DEBUG, "sent: %s\r\n", buf);
 
-  status = stream_sequential_write (ostream, buf, strlen (buf));
+  mu_stream_sequential_write (ostream, buf, strlen (buf));
   free (buf);
-  free (tempbuf);
+  mu_stream_sequential_write (ostream, "\r\n", 2);
 
   /* Reset the state.  */
   if (rc == RESP_OK)
@@ -463,7 +504,7 @@ imap4d_readline (void)
       int rc;
       
       alarm (daemon_param.timeout);
-      rc = stream_sequential_readline (istream, buffer, sizeof (buffer), &sz);
+      rc = mu_stream_sequential_readline (istream, buffer, sizeof (buffer), &sz);
       if (sz == 0)
 	{
 	  syslog (LOG_INFO, _("Unexpected eof on input"));
@@ -472,7 +513,7 @@ imap4d_readline (void)
       else if (rc)
 	{
 	  const char *p;
-	  if (stream_strerror (istream, &p))
+	  if (mu_stream_strerror (istream, &p))
 	    p = strerror (errno);
 
 	  syslog (LOG_INFO, _("Error reading from input file: %s"), p);
@@ -692,7 +733,7 @@ util_parse_822_date (char *date, time_t * timep)
   mu_timezone tz;
   const char *p = date;
 
-  if (parse822_date_time (&p, date + strlen (date), &tm, &tz) == 0)
+  if (mu_parse822_date_time (&p, date + strlen (date), &tm, &tz) == 0)
     {
       *timep = mu_tm2time (&tm, &tz);
       return 0;
@@ -793,13 +834,13 @@ util_type_to_attribute (int type, char **attr_str)
 }
 
 void
-util_print_flags (attribute_t attr)
+util_print_flags (mu_attribute_t attr)
 {
   int i;
   int flags = 0;
   int space = 0;
 
-  attribute_get_flags (attr, &flags);
+  mu_attribute_get_flags (attr, &flags);
   for (i = 0; i < _imap4d_nattr; i++)
     if (flags & _imap4d_attrlist[i].flag)
       {
@@ -819,11 +860,11 @@ util_print_flags (attribute_t attr)
 }
 
 int
-util_attribute_matches_flag (attribute_t attr, const char *item)
+util_attribute_matches_flag (mu_attribute_t attr, const char *item)
 {
   int flags = 0, mask = 0;
 
-  attribute_get_flags (attr, &flags);
+  mu_attribute_get_flags (attr, &flags);
   util_attribute_to_type (item, &mask);
   if (mask == MU_ATTRIBUTE_RECENT)
     return MU_ATTRIBUTE_IS_UNSEEN (flags);
@@ -1043,16 +1084,16 @@ util_wcard_match (const char *string, const char *pattern, const char *delim)
    first sees if it is asked to operate upon an already opened mailbox
    and if so, returns the previously computed value. */
 int
-util_uidvalidity (mailbox_t smbox, unsigned long *uidvp)
+util_uidvalidity (mu_mailbox_t smbox, unsigned long *uidvp)
 {
-  url_t mbox_url = NULL;
-  url_t smbox_url = NULL;
+  mu_url_t mbox_url = NULL;
+  mu_url_t smbox_url = NULL;
 
-  mailbox_get_url (mbox, &mbox_url);
-  mailbox_get_url (smbox, &smbox_url);
-  if (strcmp (url_to_string (mbox_url), url_to_string (smbox_url)) == 0)
+  mu_mailbox_get_url (mbox, &mbox_url);
+  mu_mailbox_get_url (smbox, &smbox_url);
+  if (strcmp (mu_url_to_string (mbox_url), mu_url_to_string (smbox_url)) == 0)
     smbox = mbox;
-  return mailbox_uidvalidity (smbox, uidvp);
+  return mu_mailbox_uidvalidity (smbox, uidvp);
 }
 
 
@@ -1064,31 +1105,31 @@ util_setio (FILE *in, FILE *out)
 
   setvbuf (in, NULL, _IOLBF, 0);
   setvbuf (out, NULL, _IOLBF, 0);
-  if (stdio_stream_create (&istream, in, MU_STREAM_NO_CLOSE)
-      || stdio_stream_create (&ostream, out, MU_STREAM_NO_CLOSE))
+  if (mu_stdio_stream_create (&istream, in, MU_STREAM_NO_CLOSE)
+      || mu_stdio_stream_create (&ostream, out, MU_STREAM_NO_CLOSE))
     imap4d_bye (ERR_NO_OFILE);
 }
 
 void
-util_get_input (stream_t *pstr)
+util_get_input (mu_stream_t *pstr)
 {
   *pstr = istream;
 }
 
 void
-util_get_output (stream_t *pstr)
+util_get_output (mu_stream_t *pstr)
 {
   *pstr = ostream;
 }
 
 void
-util_set_input (stream_t str)
+util_set_input (mu_stream_t str)
 {
   istream = str;
 }
 
 void
-util_set_output (stream_t str)
+util_set_output (mu_stream_t str)
 {
   ostream = str;
 }
@@ -1106,7 +1147,7 @@ util_wait_input (int timeout)
   
   tv.tv_sec = timeout;
   tv.tv_usec = 0;
-  status = stream_wait (istream, &wflags, &tv);
+  status = mu_stream_wait (istream, &wflags, &tv);
   if (status)
     {
       syslog (LOG_ERR, _("Cannot poll input stream: %s"),
@@ -1119,7 +1160,7 @@ util_wait_input (int timeout)
 void
 util_flush_output ()
 {
-  stream_flush (ostream);
+  mu_stream_flush (ostream);
 }
 
 int
@@ -1132,17 +1173,17 @@ util_is_master ()
 int
 imap4d_init_tls_server ()
 {
-  stream_t stream;
+  mu_stream_t stream;
   int rc;
  
-  rc = tls_stream_create (&stream, istream, ostream, 0);
+  rc = mu_tls_stream_create (&stream, istream, ostream, 0);
   if (rc)
     return 0;
 
-  if (stream_open (stream))
+  if (mu_stream_open (stream))
     {
       const char *p;
-      stream_strerror (stream, &p);
+      mu_stream_strerror (stream, &p);
       syslog (LOG_ERR, _("Cannot open TLS stream: %s"), p);
       return 0;
     }
@@ -1152,14 +1193,14 @@ imap4d_init_tls_server ()
 }
 #endif /* WITH_TLS */
 
-static list_t atexit_list;
+static mu_list_t atexit_list;
 
 void
 util_atexit (void (*fp) (void))
 {
   if (!atexit_list)
-    list_create (&atexit_list);
-  list_append (atexit_list, (void*)fp);
+    mu_list_create (&atexit_list);
+  mu_list_append (atexit_list, (void*)fp);
 }
 
 static int
@@ -1174,30 +1215,30 @@ util_bye ()
 {
   int rc = istream != ostream;
   
-  stream_close (istream);
-  stream_destroy (&istream, stream_get_owner (istream));
+  mu_stream_close (istream);
+  mu_stream_destroy (&istream, mu_stream_get_owner (istream));
 
   if (rc)
     {
-      stream_close (ostream);
-      stream_destroy (&ostream, stream_get_owner (ostream));
+      mu_stream_close (ostream);
+      mu_stream_destroy (&ostream, mu_stream_get_owner (ostream));
     }
       
-  list_do (atexit_list, atexit_run, 0);
+  mu_list_do (atexit_list, atexit_run, 0);
 }
 
 struct state_event {
   int old_state;
   int new_state;
-  list_action_t *action;
+  mu_list_action_t *action;
   void *data;
 };
 
-static list_t event_list;
+static mu_list_t event_list;
 
 void
 util_register_event (int old_state, int new_state,
-		     list_action_t *action, void *data)
+		     mu_list_action_t *action, void *data)
 {
   struct state_event *evp = malloc (sizeof (*evp));
   if (!evp)
@@ -1207,14 +1248,14 @@ util_register_event (int old_state, int new_state,
   evp->action = action;
   evp->data = data;
   if (!event_list)
-    list_create (&event_list);
-  list_append (event_list, (void*)evp);
+    mu_list_create (&event_list);
+  mu_list_append (event_list, (void*)evp);
 }
 
 void
 util_event_remove (void *id)
 {
-  list_remove (event_list, id);
+  mu_list_remove (event_list, id);
 }
 
 static int
@@ -1233,19 +1274,27 @@ util_run_events (int old_state, int new_state)
   if (event_list)
     {
       struct state_event ev;
-      iterator_t itr;
+      mu_iterator_t itr;
       ev.old_state = old_state;
       ev.new_state = new_state;
 
-      list_get_iterator (event_list, &itr);
-      for (iterator_first (itr); !iterator_is_done (itr); iterator_next (itr))
+      mu_list_get_iterator (event_list, &itr);
+      for (mu_iterator_first (itr); !mu_iterator_is_done (itr); mu_iterator_next (itr))
 	{
 	  struct state_event *p;
-	  iterator_current (itr, (void **)&p);
+	  mu_iterator_current (itr, (void **)&p);
 	  if (event_exec (p, &ev))
 	    break;
 	}
-      iterator_destroy (&itr);
+      mu_iterator_destroy (&itr);
     }
 }
   
+void
+util_chdir (const char *homedir)
+{
+  int rc = chdir (homedir);
+  if (rc)
+    mu_error ("Cannot change to home directory `%s': %s",
+	      homedir, mu_strerror (errno));
+}
