@@ -12,9 +12,9 @@
 ;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;;;; GNU General Public License for more details.
 ;;;; 
-;;;; You should have received a copy of the GNU General Public License
-;;;; along with GNU Mailutils; if not, write to the Free Software
-;;;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+;;;; You should have received a copy of the GNU General Public License along
+;;;; with GNU Mailutils; if not, write to the Free Software Foundation,
+;;;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 ;;;;
 
 (define-module (mailutils))
@@ -27,7 +27,7 @@
 		      "libmu_pop"
 		      "libmu_imap"))
 
-(let ((lib-path "/home/gray/alpha/lib/"))
+(let ((lib-path "/usr/local/lib/"))
   (for-each
    (lambda (lib)
 	   (dynamic-link (string-append lib-path lib)))
@@ -38,10 +38,10 @@
     (dynamic-call "mu_scm_init"
 		  (dynamic-link (string-append
 				 lib-path
-				 "libguile-mailutils-v-0.6.1"))))
+				 "libguile-mailutils-v-0.6.93"))))
    (else
     (load-extension (string-append
-		     lib-path "libguile-mailutils-v-0.6.1") "mu_scm_init"))))
+		     lib-path "libguile-mailutils-v-0.6.93") "mu_scm_init"))))
 
 (export mu-package)
 (export mu-package-string)
@@ -95,6 +95,8 @@
 (export mu-body-write)
 
 
+(export mu-mail-directory)
+(export mu-folder-directory)
 (export mu-mailbox-open)
 (export mu-mailbox-close)
 (export mu-mailbox-get-url)
