@@ -21,10 +21,10 @@
 long mailbox_tag;
 
 /* NOTE: Maybe will have to add some more members. That's why it is a
-   struct, not just a typedef mailbox_t */
+   struct, not just a typedef mu_mailbox_t */
 struct mu_mailbox
 {
-  mailbox_t mbox;       /* Mailbox */
+  mu_mailbox_t mbox;       /* Mailbox */
 };
 
 /* SMOB functions: */
@@ -38,11 +38,11 @@ static scm_sizet
 mu_scm_mailbox_free (SCM mailbox_smob)
 {
   struct mu_mailbox *mum = (struct mu_mailbox *) SCM_CDR (mailbox_smob);
-  mailbox_close (mum->mbox);
-  mailbox_destroy (&mum->mbox);
+  mu_mailbox_close (mum->mbox);
+  mu_mailbox_destroy (&mum->mbox);
   free (mum);
   /* NOTE: Currently there is no way for this function to return the
-     amount of memory *actually freed* by mailbox_destroy */
+     amount of memory *actually freed* by mu_mailbox_destroy */
   return sizeof (struct mu_mailbox);
 }
 
@@ -51,10 +51,10 @@ mu_scm_mailbox_print (SCM mailbox_smob, SCM port, scm_print_state * pstate)
 {
   struct mu_mailbox *mum = (struct mu_mailbox *) SCM_CDR (mailbox_smob);
   size_t count = 0;
-  url_t url = NULL;
+  mu_url_t url = NULL;
 
-  mailbox_messages_count (mum->mbox, &count);
-  mailbox_get_url (mum->mbox, &url);
+  mu_mailbox_messages_count (mum->mbox, &count);
+  mu_mailbox_get_url (mum->mbox, &url);
 
   scm_puts ("#<mailbox ", port);
 
@@ -65,7 +65,7 @@ mu_scm_mailbox_print (SCM mailbox_smob, SCM port, scm_print_state * pstate)
     }
   else
     {
-      const char *p = url_to_string (url);
+      const char *p = mu_url_to_string (url);
       if (p)
 	{
 	  char buf[64];
@@ -86,7 +86,7 @@ mu_scm_mailbox_print (SCM mailbox_smob, SCM port, scm_print_state * pstate)
 /* Internal functions */
 
 SCM
-mu_scm_mailbox_create (mailbox_t mbox)
+mu_scm_mailbox_create (mu_mailbox_t mbox)
 {
   struct mu_mailbox *mum;
 
@@ -104,12 +104,42 @@ mu_scm_is_mailbox (SCM scm)
 /* ************************************************************************* */
 /* Guile primitives */
 
-SCM_DEFINE (mu_mailbox_open, "mu-mailbox-open", 2, 0, 0,
+SCM_DEFINE (scm_mu_mail_directory, "mu-mail-directory", 0, 1, 0,
+	    (SCM URL), 
+            "")
+#define FUNC_NAME s_scm_mu_mail_directory
+{
+  if (!SCM_UNBNDP (URL))
+    {
+      SCM_ASSERT (SCM_NIMP (URL) && SCM_STRINGP (URL),
+		  URL, SCM_ARG1, FUNC_NAME);
+      mu_set_mail_directory (SCM_STRING_CHARS (URL));
+    }
+  return scm_makfrom0str (mu_mail_directory ());
+}
+#undef FUNC_NAME 
+
+SCM_DEFINE (scm_mu_folder_directory, "mu-folder-directory", 0, 1, 0,
+	    (SCM URL), 
+            "")
+#define FUNC_NAME s_scm_mu_folder_directory
+{
+  if (!SCM_UNBNDP (URL))
+    {
+      SCM_ASSERT (SCM_NIMP (URL) && SCM_STRINGP (URL),
+		  URL, SCM_ARG1, FUNC_NAME);
+      mu_set_folder_directory (SCM_STRING_CHARS (URL));
+    }
+  return scm_makfrom0str (mu_folder_directory ());
+}
+#undef FUNC_NAME 
+
+SCM_DEFINE (scm_mu_mailbox_open, "mu-mailbox-open", 2, 0, 0,
 	    (SCM URL, SCM MODE), 
             "Opens a mailbox specified by URL.")
-#define FUNC_NAME s_mu_mailbox_open
+#define FUNC_NAME s_scm_mu_mailbox_open
 {
-  mailbox_t mbox = NULL;
+  mu_mailbox_t mbox = NULL;
   char *mode_str;
   int mode = 0;
 
@@ -137,12 +167,12 @@ SCM_DEFINE (mu_mailbox_open, "mu-mailbox-open", 2, 0, 0,
   if (mode & MU_STREAM_READ && mode & MU_STREAM_WRITE)
     mode = (mode & ~(MU_STREAM_READ | MU_STREAM_WRITE)) | MU_STREAM_RDWR;
 
-  if (mailbox_create_default (&mbox, SCM_STRING_CHARS (URL)) != 0)
+  if (mu_mailbox_create_default (&mbox, SCM_STRING_CHARS (URL)) != 0)
     return SCM_BOOL_F;
 
-  if (mailbox_open (mbox, mode) != 0)
+  if (mu_mailbox_open (mbox, mode) != 0)
     {
-      mailbox_destroy (&mbox);
+      mu_mailbox_destroy (&mbox);
       return SCM_BOOL_F;
     }
 
@@ -150,64 +180,64 @@ SCM_DEFINE (mu_mailbox_open, "mu-mailbox-open", 2, 0, 0,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_close, "mu-mailbox-close", 1, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_close, "mu-mailbox-close", 1, 0, 0,
 	    (SCM MBOX), "Closes mailbox MBOX")
-#define FUNC_NAME s_mu_mailbox_close
+#define FUNC_NAME s_scm_mu_mailbox_close
 {
   struct mu_mailbox *mum;
 
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
-  mailbox_close (mum->mbox);
-  mailbox_destroy (&mum->mbox);
+  mu_mailbox_close (mum->mbox);
+  mu_mailbox_destroy (&mum->mbox);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_get_url, "mu-mailbox-get-url", 1, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_get_url, "mu-mailbox-get-url", 1, 0, 0,
 	    (SCM MBOX), 
             "Returns the URL of the mailbox.")
-#define FUNC_NAME s_mu_mailbox_get_url
+#define FUNC_NAME s_scm_mu_mailbox_get_url
 {
   struct mu_mailbox *mum;
-  url_t url;
+  mu_url_t url;
   
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
-  mailbox_get_url (mum->mbox, &url);
-  return scm_makfrom0str (url_to_string (url));
+  mu_mailbox_get_url (mum->mbox, &url);
+  return scm_makfrom0str (mu_url_to_string (url));
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_get_port, "mu-mailbox-get-port", 2, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_get_port, "mu-mailbox-get-port", 2, 0, 0,
 	    (SCM MBOX, SCM MODE),
 	    "Returns a port associated with the contents of the MBOX.\n"
 	    "MODE is a string defining operation mode of the stream. It may\n"
 	    "contain any of the two characters: \"r\" for reading, \"w\" for\n"
 	    "writing.\n")
-#define FUNC_NAME s_mu_mailbox_get_port
+#define FUNC_NAME s_scm_mu_mailbox_get_port
 {
   struct mu_mailbox *mum;
-  stream_t stream;
+  mu_stream_t stream;
   
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (SCM_NIMP (MODE) && SCM_STRINGP (MODE),
 	      MODE, SCM_ARG2, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
-  if (mailbox_get_stream (mum->mbox, &stream))
+  if (mu_mailbox_get_stream (mum->mbox, &stream))
     return SCM_BOOL_F;
   return mu_port_make_from_stream (MBOX, stream,
 				   scm_mode_bits (SCM_STRING_CHARS (MODE)));    
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_get_message, "mu-mailbox-get-message", 2, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_get_message, "mu-mailbox-get-message", 2, 0, 0,
 	    (SCM MBOX, SCM MSGNO), "Retrieve from MBOX message # MSGNO.")
-#define FUNC_NAME s_mu_mailbox_get_message
+#define FUNC_NAME s_scm_mu_mailbox_get_message
 {
   size_t msgno;
   struct mu_mailbox *mum;
-  message_t msg;
+  mu_message_t msg;
 
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT ((SCM_IMP (MSGNO) && SCM_INUMP (MSGNO)),
@@ -216,16 +246,16 @@ SCM_DEFINE (mu_mailbox_get_message, "mu-mailbox-get-message", 2, 0, 0,
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
   msgno = SCM_INUM (MSGNO);
 
-  if (mailbox_get_message (mum->mbox, msgno, &msg))
+  if (mu_mailbox_get_message (mum->mbox, msgno, &msg))
     return SCM_BOOL_F;
 
   return mu_scm_message_create (MBOX, msg);
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_messages_count, "mu-mailbox-messages-count", 1, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_messages_count, "mu-mailbox-messages-count", 1, 0, 0,
 	    (SCM MBOX), "Returns number of messages in the mailbox.")
-#define FUNC_NAME s_mu_mailbox_messages_count
+#define FUNC_NAME s_scm_mu_mailbox_messages_count
 {
   struct mu_mailbox *mum;
   size_t nmesg;
@@ -233,52 +263,52 @@ SCM_DEFINE (mu_mailbox_messages_count, "mu-mailbox-messages-count", 1, 0, 0,
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
 
-  if (mailbox_messages_count (mum->mbox, &nmesg))
+  if (mu_mailbox_messages_count (mum->mbox, &nmesg))
     return SCM_BOOL_F;
-  return scm_makenum (nmesg);
+  return mu_scm_makenum (nmesg);
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_expunge, "mu-mailbox-expunge", 1, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_expunge, "mu-mailbox-expunge", 1, 0, 0,
 	    (SCM MBOX), "Expunges deleted messages from the mailbox.")
-#define FUNC_NAME s_mu_mailbox_expunge
+#define FUNC_NAME s_scm_mu_mailbox_expunge
 {
   struct mu_mailbox *mum;
 
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
-  if (mailbox_expunge (mum->mbox))
+  if (mu_mailbox_expunge (mum->mbox))
     return SCM_BOOL_F;
   return SCM_BOOL_T;
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_url, "mu-mailbox-url", 1, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_url, "mu-mailbox-url", 1, 0, 0,
 	    (SCM MBOX), "Returns the URL of the mailbox")
-#define FUNC_NAME s_mu_mailbox_url
+#define FUNC_NAME s_scm_mu_mailbox_url
 {
   struct mu_mailbox *mum;
-  url_t url;
+  mu_url_t url;
 
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
-  mailbox_get_url (mum->mbox, &url);
-  return scm_makfrom0str (url_to_string (url));
+  mu_mailbox_get_url (mum->mbox, &url);
+  return scm_makfrom0str (mu_url_to_string (url));
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (mu_mailbox_append_message, "mu-mailbox-append-message", 2, 0, 0,
+SCM_DEFINE (scm_mu_mailbox_append_message, "mu-mailbox-append-message", 2, 0, 0,
 	    (SCM MBOX, SCM MESG), "Appends the message to the mailbox")
-#define FUNC_NAME s_mu_mailbox_append_message
+#define FUNC_NAME s_scm_mu_mailbox_append_message
 {
   struct mu_mailbox *mum;
-  message_t msg;
+  mu_message_t msg;
 
   SCM_ASSERT (mu_scm_is_mailbox (MBOX), MBOX, SCM_ARG1, FUNC_NAME);
   SCM_ASSERT (mu_scm_is_message (MESG), MESG, SCM_ARG2, FUNC_NAME);
   mum = (struct mu_mailbox *) SCM_CDR (MBOX);
   msg = mu_scm_message_get (MESG);
-  if (mailbox_append_message (mum->mbox, msg))
+  if (mu_mailbox_append_message (mum->mbox, msg))
     return SCM_BOOL_F;
   return SCM_BOOL_T;
 }
