@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -23,7 +23,7 @@
 #endif
 
 SCM
-scm_makenum (unsigned long val)
+mu_scm_makenum (unsigned long val)
 #ifndef HAVE_SCM_LONG2NUM
 {
   if (SCM_FIXABLE ((long) val))
@@ -52,26 +52,24 @@ SCM _mu_scm_package;        /* STRING: PACKAGE */
 SCM _mu_scm_version;        /* STRING: VERSION */
 SCM _mu_scm_mailer;         /* STRING: Default mailer path. */
 SCM _mu_scm_debug;          /* NUM: Default debug level. */
-SCM _mu_scm_path_maildir;   /* STRING: mu_path_maildir */
-SCM _mu_scm_path_folder_dir;/* STRING: mu_path_folder_dir */
 
 struct format_record {
   char *name;
-  record_t *record;
+  mu_record_t *record;
 };
 
 static struct format_record format_table[] = {
-  { "mbox", &mbox_record },
-  { "mh",   &mh_record },
-  { "maildir", &maildir_record },
-  { "pop",  &pop_record },
-  { "imap", &imap_record },
-  { "sendmail", &sendmail_record },
-  { "smtp", &smtp_record },
+  { "mbox", &mu_mbox_record },
+  { "mh",   &mu_mh_record },
+  { "maildir", &mu_maildir_record },
+  { "pop",  &mu_pop_record },
+  { "imap", &mu_imap_record },
+  { "sendmail", &mu_sendmail_record },
+  { "smtp", &mu_smtp_record },
   { NULL, NULL },
 };
 
-static record_t *
+static mu_record_t *
 find_format (const struct format_record *table, const char *name)
 {
   for (; table->name; table++)
@@ -84,24 +82,18 @@ static int
 register_format (const char *name)
 {
   int status = 0;
-  list_t reglist = NULL;
-  
-  status = registrar_get_list (&reglist);
-  if (status)
-    return status;
   
   if (!name)
     {
       struct format_record *table;
       for (table = format_table; table->name; table++)
-	list_append (reglist, *table->record);
-      status = 0;
+	mu_registrar_record (*table->record);
     }
   else
     {
-      record_t *record = find_format (format_table, name);
+      mu_record_t *record = find_format (format_table, name);
       if (record)
-	status = list_append (reglist, *record);
+	status = mu_registrar_record (*record);
       else
 	status = EINVAL;
     }
@@ -109,7 +101,7 @@ register_format (const char *name)
 }
     
 
-SCM_DEFINE (mu_register_format, "mu-register-format", 0, 0, 1,
+SCM_DEFINE (scm_mu_register_format, "mu-register-format", 0, 0, 1,
 	    (SCM REST),
 "Registers desired mailutils formats. Takes any number of arguments.\n"
 "Allowed arguments are:\n"
@@ -121,7 +113,7 @@ SCM_DEFINE (mu_register_format, "mu-register-format", 0, 0, 1,
 "  \"smtp\"       smtp mailer\n"
 "\n"
 "If called without arguments, registers all available formats\n")
-#define FUNC_NAME s_mu_register_format
+#define FUNC_NAME s_scm_mu_register_format
 {
   SCM status;
 
@@ -167,12 +159,11 @@ void
 mu_scm_init ()
 {
   int i;
-  list_t lst;
 
   _mu_scm_mailer = scm_makfrom0str ("sendmail:" _PATH_SENDMAIL);
   mu_set_variable ("mu-mailer", _mu_scm_mailer);
 
-  _mu_scm_debug = scm_makenum(0);
+  _mu_scm_debug = mu_scm_makenum(0);
   mu_set_variable ("mu-debug", _mu_scm_debug);
 
   _mu_scm_package = scm_makfrom0str (PACKAGE);
@@ -184,12 +175,6 @@ mu_scm_init ()
   _mu_scm_package_string = scm_makfrom0str (PACKAGE_STRING);
   mu_set_variable ("mu-package-string", _mu_scm_package_string);
 
-  _mu_scm_path_maildir = scm_makfrom0str (mu_path_maildir);
-  mu_set_variable ("mu-path-maildir", _mu_scm_path_maildir);
-
-  _mu_scm_path_folder_dir = scm_makfrom0str (mu_path_folder_dir);
-  mu_set_variable ("mu-path-folder-dir", _mu_scm_path_folder_dir);
-  
   /* Create MU- attribute names */
   for (i = 0; attr_kw[i].name; i++)
     scm_c_define(attr_kw[i].name, SCM_MAKINUM(attr_kw[i].value));
@@ -204,6 +189,5 @@ mu_scm_init ()
   mu_scm_mime_init ();
 #include "mu_scm.x"
 
-  registrar_get_list (&lst);
-  list_append (lst, path_record);
+  mu_registrar_record (mu_path_record);
 }
