@@ -11,20 +11,21 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   You should have received a copy of the GNU Lesser General
+   Public License along with this library; if not, write to the
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA */
 
 #include "mu_scm.h"
 
-SCM_DEFINE (mu_scm_getpwuid, "mu-getpwuid", 1, 0, 0,
+SCM_DEFINE (scm_mu_getpwuid, "mu-getpwuid", 1, 0, 0,
             (SCM USER),
 	    "Look up an entry in the user database. USER can be an integer,\n"
 	    "or a string, giving the behaviour of mu_get_auth_by_uid or mu_get_auth_by_name\n"
 	    "respectively.\n"
 	    "Returns a vector with fields corresponding to those of the mu_auth_data\n"
 	    "entry in question. If no matching entry was found, returns #f.\n")
-#define FUNC_NAME s_mu_scm_getpwuid
+#define FUNC_NAME s_scm_mu_getpwuid
 {
   SCM result;
   struct mu_auth_data *entry;
