@@ -36,9 +36,7 @@
 #include <string.h>
 #include <signal.h>
 #include <regex.h>
-#ifdef USE_DBM
-# include <mu_dbm.h>
-#endif
+#include <mu_dbm.h>
 #include <mailutils/libsieve.h>
 #include <mailutils/mu_auth.h>
 
@@ -47,30 +45,71 @@
    FIXME: This is for future use, when I add :mime tag
 */
 static int
-build_mime (mime_t *pmime, message_t msg, const char *text)
+build_mime (mu_sieve_machine_t mach, mu_list_t tags, mu_mime_t *pmime,
+	    mu_message_t msg, const char *text)
 {
-  mime_t mime = NULL;
-  message_t newmsg;
-  stream_t stream;
-  size_t off = 0;
-  header_t hdr;
-  body_t body;
-  /* FIXME: charset */
-  char *header = "Content-Type: text/plain;charset=iso-8859-1\n"
-    "Content-Transfer-Encoding: 8bit\n\n";
+  mu_mime_t mime = NULL;
+  mu_message_t newmsg;
+  mu_stream_t stream, input, save_input = NULL;
+  mu_header_t hdr;
+  mu_body_t body;
+  char buf[512];
+  size_t n;
+  char *header = "Content-Type: text/plain;charset=" MU_SIEVE_CHARSET "\n"
+ 	       "Content-Transfer-Encoding: 8bit\n\n";
+  int rc;
+  
+  mu_mime_create (&mime, NULL, 0);
+  mu_message_create (&newmsg, NULL);
+  mu_message_get_body (newmsg, &body);
+  mu_body_get_stream (body, &stream);
 
-  mime_create (&mime, NULL, 0);
+  if (rc = mu_memory_stream_create (&input, 0, MU_STREAM_RDWR))
+    {
+      mu_sieve_error (mach,
+		      _("cannot create temporary stream: %s"),
+		      mu_strerror (rc));
+      return 1;
+    }
 
-  message_create (&newmsg, NULL);
-  message_get_body (newmsg, &body);
-  header_create (&hdr, header, strlen (header), newmsg);
-  message_set_header (newmsg, hdr, NULL);
-  body_get_stream (body, &stream);
+  if (rc = mu_stream_open (input))
+    {
+      mu_sieve_error (mach,
+		      _("cannot open temporary stream: %s"),
+		      mu_strerror (rc));
+      return 1;
+    }
+  
+  mu_stream_write (input, text, strlen (text), 0, &n);
 
-  stream_printf (stream, &off, "%s", text);
-  stream_close (stream);
-  mime_add_part (mime, newmsg);
-  message_unref (newmsg);
+  if (mu_sieve_tag_lookup (tags, "mime", NULL))
+    {
+      mu_stream_t fstr;
+      rc = mu_filter_create (&fstr, input, "base64",
+			     MU_FILTER_ENCODE, MU_STREAM_READ);
+      if (rc == 0) 
+	{
+	  header = "Content-Type: text/plain;charset=" MU_SIEVE_CHARSET "\n"
+	           "Content-Transfer-Encoding: base64\n\n";
+	  save_input = input;
+	  input = fstr;
+	}
+    }
+
+  while (rc == 0
+	 && mu_stream_sequential_read (input, buf, sizeof buf, &n) == 0
+	 && n > 0)
+    rc = mu_stream_sequential_write (stream, buf, n);
+
+  mu_stream_destroy (&input, mu_stream_get_owner (input));
+  if (save_input)
+    mu_stream_destroy (&save_input, mu_stream_get_owner (save_input));
+  
+  mu_header_create (&hdr, header, strlen (header), newmsg);
+  mu_message_set_header (newmsg, hdr, NULL);
+
+  mu_mime_add_part (mime, newmsg);
+  mu_message_unref (newmsg);
 
   *pmime = mime;
 
@@ -81,24 +120,24 @@ build_mime (mime_t *pmime, message_t msg, const char *text)
 
 /* Produce diagnostic output. */
 static int
-diag (sieve_machine_t mach)
+diag (mu_sieve_machine_t mach)
 {
-  if (sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
+  if (mu_sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
     {
-      sieve_locus_t locus;
-      sieve_get_locus (mach, &locus);
-      sieve_debug (mach, "%s:%lu: VACATION\n",
+      mu_sieve_locus_t locus;
+      mu_sieve_get_locus (mach, &locus);
+      mu_sieve_debug (mach, "%s:%lu: VACATION\n",
 		   locus.source_file,
 		   (unsigned long) locus.source_line);
     }
 
-  sieve_log_action (mach, "VACATION", NULL);
-  return sieve_is_dry_run (mach);
+  mu_sieve_log_action (mach, "VACATION", NULL);
+  return mu_sieve_is_dry_run (mach);
 }
 
 
 struct addr_data {
-  address_t addr;
+  mu_address_t addr;
   char *my_address;
 };
 
@@ -106,7 +145,7 @@ static int
 _compare (void *item, void *data)
 {
   struct addr_data *ad = data;
-  int rc = address_contains_email (ad->addr, item);
+  int rc = mu_address_contains_email (ad->addr, item);
   if (rc)
     ad->my_address = item;
   return rc;
@@ -116,29 +155,29 @@ _compare (void *item, void *data)
    of the originating mail. Return non-zero if so and store a pointer
    to the matching address to *MY_ADDRESS. */
 static int
-match_addresses (header_t hdr, sieve_value_t *addresses, char **my_address)
+match_addresses (mu_header_t hdr, mu_sieve_value_t *addresses, char **my_address)
 {
   int match = 0;
   char *str;
   struct addr_data ad;
 
   ad.my_address = NULL;
-  if (header_aget_value (hdr, MU_HEADER_TO, &str) == 0)
+  if (mu_header_aget_value (hdr, MU_HEADER_TO, &str) == 0)
     {
-      if (!address_create (&ad.addr, str))
+      if (!mu_address_create (&ad.addr, str))
 	{
-	  match += sieve_vlist_do (addresses, _compare, &ad);
-	  address_destroy (&ad.addr);
+	  match += mu_sieve_vlist_do (addresses, _compare, &ad);
+	  mu_address_destroy (&ad.addr);
 	}
       free (str);
     }
 
-  if (!match && header_aget_value (hdr, MU_HEADER_CC, &str) == 0)
+  if (!match && mu_header_aget_value (hdr, MU_HEADER_CC, &str) == 0)
     {
-      if (!address_create (&ad.addr, str))
+      if (!mu_address_create (&ad.addr, str))
 	{
-	  match += sieve_vlist_do (addresses, _compare, &ad);
-	  address_destroy (&ad.addr);
+	  match += mu_sieve_vlist_do (addresses, _compare, &ad);
+	  mu_address_destroy (&ad.addr);
 	}
       free (str);
     }
@@ -148,7 +187,7 @@ match_addresses (header_t hdr, sieve_value_t *addresses, char **my_address)
 
 
 struct regex_data {
-  sieve_machine_t mach;
+  mu_sieve_machine_t mach;
   char *email;
 };
 
@@ -162,10 +201,10 @@ regex_comparator (void *item, void *data)
   if (regcomp (&preg, item,
 	       REG_EXTENDED | REG_NOSUB | REG_NEWLINE | REG_ICASE))
     {
-      sieve_error (d->mach,
-		   ("%d: cannot compile regular expression \"%s\"\n"),
-		   sieve_get_message_num (d->mach),
-		   item);
+      mu_sieve_error (d->mach,
+		      _("%d: cannot compile regular expression \"%s\"\n"),
+		      mu_sieve_get_message_num (d->mach),
+		      item);
       return 0;
     }
   rc = regexec (&preg, d->email, 0, NULL, 0) == 0;
@@ -176,10 +215,10 @@ regex_comparator (void *item, void *data)
 /* Decide whether EMAIL address should not be responded to.
  */
 static int
-noreply_address_p (sieve_machine_t mach, list_t tags, char *email)
+noreply_address_p (mu_sieve_machine_t mach, mu_list_t tags, char *email)
 {
   int i, rc = 0;
-  sieve_value_t *arg;
+  mu_sieve_value_t *arg;
   struct regex_data rd;
   static char *noreply_sender[] = {
     ".*-REQUEST@.*",
@@ -198,8 +237,8 @@ noreply_address_p (sieve_machine_t mach, list_t tags, char *email)
   for (i = 0; rc == 0 && noreply_sender[i]; i++)
     rc = regex_comparator (noreply_sender[i], &rd);
 
-  if (!rc && sieve_tag_lookup (tags, "addresses", &arg))
-    rc = sieve_vlist_do (arg, regex_comparator, &rd);
+  if (!rc && mu_sieve_tag_lookup (tags, "addresses", &arg))
+    rc = mu_sieve_vlist_do (arg, regex_comparator, &rd);
   
   return rc;
 }
@@ -207,11 +246,11 @@ noreply_address_p (sieve_machine_t mach, list_t tags, char *email)
 
 /* Return T if letter precedence is 'bulk' or 'junk' */
 static int
-bulk_precedence_p (header_t hdr)
+bulk_precedence_p (mu_header_t hdr)
 {
   int rc = 0;
   char *str;
-  if (header_aget_value (hdr, MU_HEADER_PRECEDENCE, &str) == 0)
+  if (mu_header_aget_value (hdr, MU_HEADER_PRECEDENCE, &str) == 0)
     {
       rc = strcasecmp (str, "bulk") == 0
 	   || strcasecmp (str, "junk") == 0;
@@ -227,7 +266,7 @@ bulk_precedence_p (header_t hdr)
 /* Check and updated vacation database. Return 0 if the mail should
    be answered. */
 static int
-check_db (sieve_machine_t mach, list_t tags, char *from)
+check_db (mu_sieve_machine_t mach, mu_list_t tags, char *from)
 {
 #ifdef USE_DBM
   DBM_FILE db;
@@ -235,11 +274,11 @@ check_db (sieve_machine_t mach, list_t tags, char *from)
   time_t now;
   char buffer[64];
   char *file, *home;
-  sieve_value_t *arg;
+  mu_sieve_value_t *arg;
   unsigned int days;
   int rc;
   
-  if (sieve_tag_lookup (tags, "days", &arg))
+  if (mu_sieve_tag_lookup (tags, "days", &arg))
     {
       days = arg->v.number;
       if (days < DAYS_MIN)
@@ -254,22 +293,22 @@ check_db (sieve_machine_t mach, list_t tags, char *from)
 
   if (asprintf (&file, "%s/.vacation", (home ? home : ".")) == -1)
     {
-      sieve_error (mach, _("%d: cannot build db file name"),
-		   sieve_get_message_num (mach));
+      mu_sieve_error (mach, _("%d: cannot build db file name"),
+		      mu_sieve_get_message_num (mach));
       free (home);
-      sieve_abort (mach);
+      mu_sieve_abort (mach);
     }
   free (home);
   
   rc = mu_dbm_open (file, &db, MU_STREAM_RDWR, 0600);
   if (rc)
     {
-      sieve_error (mach,
-		   _("%d: cannot open `%s': %s"),
-		   sieve_get_message_num (mach), file,
-		   mu_strerror (rc));
+      mu_sieve_error (mach,
+		      _("%d: cannot open `%s': %s"),
+		      mu_sieve_get_message_num (mach), file,
+		      mu_strerror (rc));
       free (file);
-      sieve_abort (mach);
+      mu_sieve_abort (mach);
     }
   free (file);
 
@@ -305,11 +344,11 @@ check_db (sieve_machine_t mach, list_t tags, char *from)
   mu_dbm_close (db);
   return 0;
 #else
-  sieve_error (mach,
+  mu_sieve_error (mach,
 	       /* TRANSLATORS: 'vacation' and ':days' are Sieve keywords.
 		  Do not translate them! */
 	       _("%d: vacation compiled without DBM support. Ignoring :days tag"),
-	       sieve_get_message_num (mach));
+	       mu_sieve_get_message_num (mach));
   return 0;
 #endif
 }
@@ -321,13 +360,13 @@ check_db (sieve_machine_t mach, list_t tags, char *from)
    "reply_prefix" tag.
  */
 static void
-re_subject (sieve_machine_t mach, list_t tags, char **psubject)
+re_subject (mu_sieve_machine_t mach, mu_list_t tags, char **psubject)
 {
   char *subject;
-  sieve_value_t *arg;
+  mu_sieve_value_t *arg;
   char *prefix = "Re";
   
-  if (sieve_tag_lookup (tags, "reply_prefix", &arg))
+  if (mu_sieve_tag_lookup (tags, "reply_prefix", &arg))
     {
       prefix = arg->v.string;
     }
@@ -335,9 +374,9 @@ re_subject (sieve_machine_t mach, list_t tags, char **psubject)
   subject = malloc (strlen (*psubject) + strlen (prefix) + 3);
   if (!subject)
     {
-      sieve_error (mach,
-		   ("%d: not enough memory"),
-		   sieve_get_message_num (mach));
+      mu_sieve_error (mach,
+		      _("%d: not enough memory"),
+		      mu_sieve_get_message_num (mach));
       return;
     }
   
@@ -355,26 +394,26 @@ re_subject (sieve_machine_t mach, list_t tags, char **psubject)
    Otherwise, reply_prefix is prepended to it. */
 
 static void
-vacation_subject (sieve_machine_t mach, list_t tags,
-		  message_t msg, header_t newhdr)
+vacation_subject (mu_sieve_machine_t mach, mu_list_t tags,
+		  mu_message_t msg, mu_header_t newhdr)
 {
-  sieve_value_t *arg;
+  mu_sieve_value_t *arg;
   char *value;
   char *subject;
   int subject_allocated;
-  header_t hdr;
+  mu_header_t hdr;
   
-  if (sieve_tag_lookup (tags, "subject", &arg))
+  if (mu_sieve_tag_lookup (tags, "subject", &arg))
     {
       subject =  arg->v.string;
       subject_allocated = 0;
     }
-  else if (message_get_header (msg, &hdr) == 0
-	   && header_aget_value_unfold (hdr, MU_HEADER_SUBJECT, &value) == 0)
+  else if (mu_message_get_header (msg, &hdr) == 0
+	   && mu_header_aget_value_unfold (hdr, MU_HEADER_SUBJECT, &value) == 0)
     {
       char *p;
       
-      int rc = rfc2047_decode ("iso-8859-1", value, &p);
+      int rc = mu_rfc2047_decode (MU_SIEVE_CHARSET, value, &p);
 
       subject_allocated = 1;
       if (rc)
@@ -387,33 +426,33 @@ vacation_subject (sieve_machine_t mach, list_t tags,
 	  subject = p;
 	}
 
-      if (sieve_tag_lookup (tags, "reply_regex", &arg))
+      if (mu_sieve_tag_lookup (tags, "reply_regex", &arg))
 	{
 	  char *err = NULL;
 	  
-	  rc = munre_set_regex (arg->v.string, 0, &err);
+	  rc = mu_unre_set_regex (arg->v.string, 0, &err);
 	  if (rc)
 	    {
-	      sieve_error (mach,
-			   ("%d: vacation - cannot compile reply prefix regexp: %s: %s"),
-			   sieve_get_message_num (mach),
-			   mu_strerror (rc),
-			   err ? err : "");
+	      mu_sieve_error (mach,
+			      _("%d: vacation - cannot compile reply prefix regexp: %s: %s"),
+			      mu_sieve_get_message_num (mach),
+			      mu_strerror (rc),
+			      err ? err : "");
 	    }
 	}
 	  
-      if (munre_subject (subject, NULL))
+      if (mu_unre_subject (subject, NULL))
 	re_subject (mach, tags, &subject);
       
       free (value);
     }
 
-  if (rfc2047_encode ("iso-8859-1", "quoted-printable",
-		      subject, &value))
-    header_set_value (newhdr, MU_HEADER_SUBJECT, subject, 0);
+  if (mu_rfc2047_encode (MU_SIEVE_CHARSET, "quoted-printable",
+			 subject, &value))
+    mu_header_set_value (newhdr, MU_HEADER_SUBJECT, subject, 0);
   else
     {
-      header_set_value (newhdr, MU_HEADER_SUBJECT, value, 0);
+      mu_header_set_value (newhdr, MU_HEADER_SUBJECT, value, 0);
       free (value);
     }
 
@@ -423,37 +462,38 @@ vacation_subject (sieve_machine_t mach, list_t tags,
 
 /* Generate and send the reply message */
 static int
-vacation_reply (sieve_machine_t mach, list_t tags, message_t msg,
+vacation_reply (mu_sieve_machine_t mach, mu_list_t tags, mu_message_t msg,
 		char *text, char *to, char *from)
 {
-  mime_t mime = NULL;
-  message_t newmsg;
-  header_t newhdr;
-  address_t to_addr = NULL, from_addr = NULL;
+  mu_mime_t mime = NULL;
+  mu_message_t newmsg;
+  mu_header_t newhdr;
+  mu_address_t to_addr = NULL, from_addr = NULL;
   char *value;
-  mailer_t mailer;
+  mu_mailer_t mailer;
   int rc;
   
-  build_mime (&mime, msg, text);
-  mime_get_message (mime, &newmsg);
-  message_get_header (newmsg, &newhdr);
+  if (build_mime (mach, tags, &mime, msg, text))
+    return -1;
+  mu_mime_get_message (mime, &newmsg);
+  mu_message_get_header (newmsg, &newhdr);
 
-  rc = address_create (&to_addr, to);
+  rc = mu_address_create (&to_addr, to);
   if (rc)
     {
-      sieve_error (mach,
-		   _("%d: cannot create recipient address <%s>: %s"),
-		   sieve_get_message_num (mach), from, mu_strerror (rc));
+      mu_sieve_error (mach,
+		      _("%d: cannot create recipient address <%s>: %s"),
+		      mu_sieve_get_message_num (mach), from, mu_strerror (rc));
       return -1;
     }
 
-  header_set_value (newhdr, MU_HEADER_TO, to, 0);
+  mu_header_set_value (newhdr, MU_HEADER_TO, to, 0);
   
   vacation_subject (mach, tags, msg, newhdr);
     
   if (from)
     {
-      if (address_create (&from_addr, from))
+      if (mu_address_create (&from_addr, from))
 	from_addr = NULL;
     }
   else
@@ -461,72 +501,79 @@ vacation_reply (sieve_machine_t mach, list_t tags, message_t msg,
       from_addr = NULL;
     }
   
-  mu_rfc2822_in_reply_to (msg, &value);
-  header_set_value (newhdr, MU_HEADER_IN_REPLY_TO, value, 1);
-  free (value);
+  if (mu_rfc2822_in_reply_to (msg, &value) == 0)
+    {
+      mu_header_set_value (newhdr, MU_HEADER_IN_REPLY_TO, value, 1);
+      free (value);
+    }
   
-  mu_rfc2822_references (msg, &value);
-  header_set_value (newhdr, MU_HEADER_REFERENCES, value, 1);
-  free (value);
+  if (mu_rfc2822_references (msg, &value) == 0)
+    {
+      mu_header_set_value (newhdr, MU_HEADER_REFERENCES, value, 1);
+      free (value);
+    }
   
-  mailer = sieve_get_mailer (mach);
-  rc = mailer_open (mailer, 0);
+  mailer = mu_sieve_get_mailer (mach);
+  rc = mu_mailer_open (mailer, 0);
   if (rc)
     {
-      url_t url = NULL;
-      mailer_get_url (mailer, &url);
+      mu_url_t url = NULL;
+      mu_mailer_get_url (mailer, &url);
       
-      sieve_error (mach,
-		   _("%d: cannot open mailer %s: %s"),
-		   sieve_get_message_num (mach),
-		   url_to_string (url), mu_strerror (rc));
+      mu_sieve_error (mach,
+		      _("%d: cannot open mailer %s: %s"),
+		      mu_sieve_get_message_num (mach),
+		      mu_url_to_string (url), mu_strerror (rc));
       return -1;
     }
-
-  rc = mailer_send_message (mailer, newmsg, from_addr, to_addr);
-  mailer_close (mailer);
+  else
+    {
+      rc = mu_mailer_send_message (mailer, newmsg, from_addr, to_addr);
+      mu_mailer_close (mailer);
+    }
+  mu_mime_destroy (&mime);
   return rc;
 }
 
 int
-sieve_action_vacation (sieve_machine_t mach, list_t args, list_t tags)
+sieve_action_vacation (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
 {
   int rc;
   char *text, *from;
-  sieve_value_t *val;
-  message_t msg;
-  header_t hdr;
-  char *my_address = sieve_get_daemon_email (mach);
+  mu_sieve_value_t *val;
+  mu_message_t msg;
+  mu_header_t hdr;
+  char *my_address = mu_sieve_get_daemon_email (mach);
   
   if (diag (mach))
     return 0;
   
-  val = sieve_value_get (args, 0);
+  val = mu_sieve_value_get (args, 0);
   if (!val)
     {
-      sieve_error (mach, _("cannot get text!"));
-      sieve_abort (mach);
+      mu_sieve_error (mach, _("cannot get text!"));
+      mu_sieve_abort (mach);
     }
   else
     text = val->v.string;
 
-  msg = sieve_get_message (mach);
-  message_get_header (msg, &hdr);
+  msg = mu_sieve_get_message (mach);
+  mu_message_get_header (msg, &hdr);
 
-  if (sieve_tag_lookup (tags, "sender", &val))
+  if (mu_sieve_tag_lookup (tags, "sender", &val))
     {
       /* Debugging hook: :sender sets fake reply address */
       from = strdup (val->v.string);
     }
-  else if (sieve_get_message_sender (msg, &from))
+  else if (mu_sieve_get_message_sender (msg, &from))
     {
-      sieve_error (mach,
-		   _("%d: cannot get sender address"),
-		   sieve_get_message_num (mach));
-      sieve_abort (mach);
+      mu_sieve_error (mach,
+		      _("%d: cannot get sender address"),
+		      mu_sieve_get_message_num (mach));
+      mu_sieve_abort (mach);
     }
 
-  if (sieve_tag_lookup (tags, "aliases", &val)
+  if (mu_sieve_tag_lookup (tags, "aliases", &val)
       && match_addresses (hdr, val, &my_address) == 0)
     return 0;
 
@@ -541,34 +588,35 @@ sieve_action_vacation (sieve_machine_t mach, list_t args, list_t tags)
   rc = vacation_reply (mach, tags, msg, text, from, my_address);
   free (from);
   if (rc == -1)
-    sieve_abort (mach);
+    mu_sieve_abort (mach);
   return rc;
 }
 
 /* Tagged arguments: */
-static sieve_tag_def_t vacation_tags[] = {
+static mu_sieve_tag_def_t vacation_tags[] = {
   {"days", SVT_NUMBER},
   {"subject", SVT_STRING},
   {"aliases", SVT_STRING_LIST},
   {"addresses", SVT_STRING_LIST},
   {"reply_regex", SVT_STRING},
   {"reply_prefix", SVT_STRING},
+  {"mime", SVT_VOID},
   {NULL}
 };
 
-static sieve_tag_group_t vacation_tag_groups[] = {
+static mu_sieve_tag_group_t vacation_tag_groups[] = {
   {vacation_tags, NULL},
   {NULL}
 };
 
 /* Required arguments: */
-static sieve_data_type vacation_args[] = {
+static mu_sieve_data_type vacation_args[] = {
   SVT_STRING,			/* message text */
   SVT_VOID
 };
 
-int SIEVE_EXPORT (vacation, init) (sieve_machine_t mach)
+int SIEVE_EXPORT (vacation, init) (mu_sieve_machine_t mach)
 {
-  return sieve_register_action (mach, "vacation", sieve_action_vacation,
+  return mu_sieve_register_action (mach, "vacation", sieve_action_vacation,
 				vacation_args, vacation_tag_groups, 1);
 }
