@@ -1,19 +1,20 @@
-/* GNU mailutils - a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 1999, 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   This program is distributed in the hope that it will be useful,
+   GNU Mailutils is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with GNU Mailutils; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "mail.h"
 
@@ -26,9 +27,9 @@
 int
 mail_write (int argc, char **argv)
 {
-  message_t msg;
-  body_t bod;
-  stream_t stream;
+  mu_message_t msg;
+  mu_body_t bod;
+  mu_stream_t stream;
   FILE *output;
   char *filename = NULL;
   msgset_t *msglist = NULL, *mp;
@@ -41,13 +42,19 @@ mail_write (int argc, char **argv)
     filename = util_outfolder_name (argv[--argc]);
   else
     {
+      size_t n = get_cursor ();
       char *p = NULL;
-      asprintf (&p, "%d", cursor);
+      if (n == 0)
+        {
+          util_error (_("No applicable message"));
+          return 1;
+        }
+      asprintf (&p, "%u", n);
       filename = util_outfolder_name (p);
       free (p);
     }
 		
-  if (msgset_parse (argc, argv, &msglist))
+  if (msgset_parse (argc, argv, MSG_NODELETED|MSG_SILENT, &msglist))
     {
       if (filename)
           free (filename);
@@ -67,7 +74,7 @@ mail_write (int argc, char **argv)
   output = fopen (filename, "a");
   if (!output)
     {
-      util_error("can't open %s: %s", filename, strerror (errno));
+      util_error (_("can't open %s: %s"), filename, strerror (errno));
       free (filename);
       fclose (output);
       msgset_free (msglist);
@@ -76,24 +83,24 @@ mail_write (int argc, char **argv)
 
   for (mp = msglist; mp; mp = mp->next)
     {
-      attribute_t attr;
+      mu_attribute_t attr;
       char buffer[512];
       off_t off = 0;
       size_t n = 0;
 
-      if (util_get_message (mbox, mp->msg_part[0], &msg, MSG_NODELETED))
+      if (util_get_message (mbox, mp->msg_part[0], &msg))
         continue;
 
-      message_get_body (msg, &bod);
+      mu_message_get_body (msg, &bod);
 
-      body_size (bod, &size);
+      mu_body_size (bod, &size);
       total_size += size;
-      body_lines (bod, &size);
+      mu_body_lines (bod, &size);
       total_lines += size;
 
-      body_get_stream (bod, &stream);
+      mu_body_get_stream (bod, &stream);
       /* should there be a separator? */
-      while (stream_read(stream, buffer, sizeof (buffer) - 1, off, &n) == 0
+      while (mu_stream_read(stream, buffer, sizeof (buffer) - 1, off, &n) == 0
 	     && n != 0)
 	{
 	  buffer[n] = '\0';
@@ -103,8 +110,8 @@ mail_write (int argc, char **argv)
 
       /* mark as saved. */
 
-      message_get_attribute (msg, &attr);
-      attribute_set_userflag (attr, MAIL_ATTRIBUTE_SAVED);
+      mu_message_get_attribute (msg, &attr);
+      mu_attribute_set_userflag (attr, MAIL_ATTRIBUTE_SAVED);
     }
 
   fprintf (ofile, "\"%s\" %3d/%-5d\n", filename, total_lines, total_size);
