@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
    2004, 2005 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
@@ -12,9 +12,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   You should have received a copy of the GNU Lesser General
+   Public License along with this library; if not, write to the
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA */
 
 /* Mailutils Abstract Mail Directory Layer 
    First draft by Sergey Poznyakoff.
@@ -69,49 +70,49 @@
 #include <url0.h>
 #include <amd.h>
 
-static void amd_destroy __P((mailbox_t mailbox));
-static int amd_open __P ((mailbox_t, int));
-static int amd_close __P ((mailbox_t));
-static int amd_get_message __P ((mailbox_t, size_t, message_t *));
-static int amd_append_message __P ((mailbox_t, message_t));
-static int amd_messages_count __P ((mailbox_t, size_t *));
-static int amd_messages_recent __P ((mailbox_t, size_t *));
-static int amd_message_unseen __P ((mailbox_t, size_t *));
-static int amd_expunge __P ((mailbox_t));
-static int amd_save_attributes __P ((mailbox_t));
-static int amd_uidnext __P((mailbox_t mailbox, size_t *puidnext));
-static int amd_uidvalidity __P ((mailbox_t, unsigned long *));
-static int amd_scan __P ((mailbox_t, size_t, size_t *));
-static int amd_is_updated __P ((mailbox_t));
-static int amd_get_size __P ((mailbox_t, off_t *));
+static void amd_destroy (mu_mailbox_t mailbox);
+static int amd_open (mu_mailbox_t, int);
+static int amd_close (mu_mailbox_t);
+static int amd_get_message (mu_mailbox_t, size_t, mu_message_t *);
+static int amd_append_message (mu_mailbox_t, mu_message_t);
+static int amd_messages_count (mu_mailbox_t, size_t *);
+static int amd_messages_recent (mu_mailbox_t, size_t *);
+static int amd_message_unseen (mu_mailbox_t, size_t *);
+static int amd_expunge (mu_mailbox_t);
+static int amd_save_attributes (mu_mailbox_t);
+static int amd_uidnext (mu_mailbox_t mailbox, size_t *puidnext);
+static int amd_uidvalidity (mu_mailbox_t, unsigned long *);
+static int amd_scan (mu_mailbox_t, size_t, size_t *);
+static int amd_is_updated (mu_mailbox_t);
+static int amd_get_size (mu_mailbox_t, mu_off_t *);
 
-static int amd_body_read __P ((stream_t, char *, size_t, off_t, size_t *));
-static int amd_body_readline __P ((stream_t, char *, size_t, off_t, size_t *));
-static int amd_stream_size __P ((stream_t stream, off_t *psize));
+static int amd_body_read (mu_stream_t, char *, size_t, mu_off_t, size_t *);
+static int amd_body_readline (mu_stream_t, char *, size_t, mu_off_t, size_t *);
+static int amd_stream_size (mu_stream_t stream, mu_off_t *psize);
 
-static int amd_body_size __P ((body_t body, size_t *psize));
-static int amd_body_lines __P ((body_t body, size_t *plines));
+static int amd_body_size (mu_body_t body, size_t *psize);
+static int amd_body_lines (mu_body_t body, size_t *plines);
 
-static int amd_header_fill __P((header_t header, char *buffer, size_t len,
-			       off_t off, size_t *pnread));
-static int amd_header_size __P((header_t header, size_t *psize));
-static int amd_header_lines __P((header_t header, size_t *plines));
+static int amd_header_fill (mu_header_t header, char *buffer, size_t len,
+			    mu_off_t off, size_t *pnread);
+static int amd_header_size (mu_header_t header, size_t *psize);
+static int amd_header_lines (mu_header_t header, size_t *plines);
 
-static int amd_get_attr_flags __P((attribute_t attr, int *pflags));
-static int amd_set_attr_flags __P((attribute_t attr, int flags));
-static int amd_unset_attr_flags __P((attribute_t attr, int flags));
+static int amd_get_attr_flags (mu_attribute_t attr, int *pflags);
+static int amd_set_attr_flags (mu_attribute_t attr, int flags);
+static int amd_unset_attr_flags (mu_attribute_t attr, int flags);
 
-static void _amd_message_delete __P((struct _amd_data *amd,
-				    struct _amd_message *msg));
-static int amd_pool_open __P((struct _amd_message *mhm));
-static int amd_pool_open_count __P((struct _amd_data *amd));
+static void _amd_message_delete (struct _amd_data *amd,
+				 struct _amd_message *msg);
+static int amd_pool_open (struct _amd_message *mhm);
+static int amd_pool_open_count (struct _amd_data *amd);
 static void amd_pool_flush (struct _amd_data *amd);
-static struct _amd_message **amd_pool_lookup __P((struct _amd_message *mhm));
+static struct _amd_message **amd_pool_lookup (struct _amd_message *mhm);
 
-static int amd_envelope_date __P((envelope_t envelope, char *buf, size_t len,
-				  size_t *psize));
-static int amd_envelope_sender __P((envelope_t envelope, char *buf, size_t len,
-				   size_t *psize));
+static int amd_envelope_date (mu_envelope_t envelope, char *buf, size_t len,
+			      size_t *psize);
+static int amd_envelope_sender (mu_envelope_t envelope, char *buf, size_t len,
+			        size_t *psize);
 
 /* Operations on message array */
 
@@ -124,11 +125,11 @@ static int amd_envelope_sender __P((envelope_t envelope, char *buf, size_t len,
    Indexes are zero-based. */
    
 static int
-amd_msg_bsearch (struct _amd_data *amd, off_t first, off_t last,
+amd_msg_bsearch (struct _amd_data *amd, mu_off_t first, mu_off_t last,
 		 struct _amd_message *msg,
-		 off_t *pret)
+		 mu_off_t *pret)
 {
-  off_t mid;
+  mu_off_t mid;
   int rc;
 
   if (last < first)
@@ -162,9 +163,9 @@ amd_msg_lookup (struct _amd_data *amd, struct _amd_message *msg,
 		 size_t *pret)
 {
   int rc;
-  off_t i;
+  mu_off_t i;
   
-  if (!amd->msg_array)
+  if (amd->msg_count == 0)
     {
       *pret = 0;
       return 1;
@@ -241,7 +242,7 @@ amd_array_shrink (struct _amd_data *amd, size_t index)
 
 
 int
-amd_init_mailbox (mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd) 
+amd_init_mailbox (mu_mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd) 
 {
   struct _amd_data *amd;
   size_t name_len;
@@ -258,7 +259,7 @@ amd_init_mailbox (mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd)
   /* Back pointer.  */
   amd->mailbox = mailbox;
 
-  url_get_path (mailbox->url, NULL, 0, &name_len);
+  mu_url_get_path (mailbox->url, NULL, 0, &name_len);
   amd->name = calloc (name_len + 1, sizeof (char));
   if (amd->name == NULL)
     {
@@ -266,7 +267,7 @@ amd_init_mailbox (mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd)
       mailbox->data = NULL;
       return ENOMEM;
     }
-  url_get_path (mailbox->url, amd->name, name_len + 1, NULL);
+  mu_url_get_path (mailbox->url, amd->name, name_len + 1, NULL);
 
   /* Overloading the defaults.  */
   mailbox->_destroy = amd_destroy;
@@ -296,7 +297,7 @@ amd_init_mailbox (mailbox_t mailbox, size_t amd_size, struct _amd_data **pamd)
 }
 
 static void
-amd_destroy (mailbox_t mailbox)
+amd_destroy (mu_mailbox_t mailbox)
 {
   struct _amd_data *amd = mailbox->data;
   size_t i;
@@ -305,10 +306,10 @@ amd_destroy (mailbox_t mailbox)
     return;
 
   amd_pool_flush (amd);
-  monitor_wrlock (mailbox->monitor);
+  mu_monitor_wrlock (mailbox->monitor);
   for (i = 0; i < amd->msg_count; i++)
     {
-      message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
+      mu_message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
       free (amd->msg_array[i]);
     }
   free (amd->msg_array);
@@ -318,11 +319,11 @@ amd_destroy (mailbox_t mailbox)
 
   free (amd);
   mailbox->data = NULL;
-  monitor_unlock (mailbox->monitor);
+  mu_monitor_unlock (mailbox->monitor);
 }
 
 static int
-amd_open (mailbox_t mailbox, int flags)
+amd_open (mu_mailbox_t mailbox, int flags)
 {
   struct _amd_data *amd = mailbox->data;
   struct stat st;
@@ -340,7 +341,7 @@ amd_open (mailbox_t mailbox, int flags)
 }
 
 static int
-amd_close (mailbox_t mailbox)
+amd_close (mu_mailbox_t mailbox)
 {
   struct _amd_data *amd;
   int i;
@@ -352,10 +353,10 @@ amd_close (mailbox_t mailbox)
   
   /* Destroy all cached data */
   amd_pool_flush (amd);
-  monitor_wrlock (mailbox->monitor);
+  mu_monitor_wrlock (mailbox->monitor);
   for (i = 0; i < amd->msg_count; i++)
     {
-      message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
+      mu_message_destroy (&amd->msg_array[i]->message, amd->msg_array[i]);
       free (amd->msg_array[i]);
     }
   free (amd->msg_array);
@@ -379,11 +380,11 @@ _amd_get_message (struct _amd_data *amd, size_t msgno)
 }
 
 static int
-_amd_attach_message (mailbox_t mailbox, struct _amd_message *mhm,
-		    message_t *pmsg)
+_amd_attach_message (mu_mailbox_t mailbox, struct _amd_message *mhm,
+		    mu_message_t *pmsg)
 {
   int status;
-  message_t msg;
+  mu_message_t msg;
 
   /* Check if we already have it.  */
   if (mhm->message)
@@ -394,87 +395,87 @@ _amd_attach_message (mailbox_t mailbox, struct _amd_message *mhm,
     }
 
   /* Get an empty message struct.  */
-  status = message_create (&msg, mhm);
+  status = mu_message_create (&msg, mhm);
   if (status != 0)
     return status;
 
   /* Set the header.  */
   {
-    header_t header = NULL;
-    status = header_create (&header, NULL, 0, msg);
+    mu_header_t header = NULL;
+    status = mu_header_create (&header, NULL, 0, msg);
     if (status != 0)
       {
-	message_destroy (&msg, mhm);
+	mu_message_destroy (&msg, mhm);
 	return status;
       }
-    header_set_fill (header, amd_header_fill, msg);
-    header_set_size (header, amd_header_size, msg);
-    header_set_lines (header, amd_header_lines, msg);
+    mu_header_set_fill (header, amd_header_fill, msg);
+    mu_header_set_size (header, amd_header_size, msg);
+    mu_header_set_lines (header, amd_header_lines, msg);
     /*FIXME:
-    header_set_get_fvalue (header, amd_header_get_fvalue, msg);
+    mu_header_set_get_fvalue (header, amd_header_get_fvalue, msg);
     */
-    message_set_header (msg, header, mhm);
+    mu_message_set_header (msg, header, mhm);
   }
 
   /* Set the attribute.  */
   {
-    attribute_t attribute;
-    status = attribute_create (&attribute, msg);
+    mu_attribute_t attribute;
+    status = mu_attribute_create (&attribute, msg);
     if (status != 0)
       {
-	message_destroy (&msg, mhm);
+	mu_message_destroy (&msg, mhm);
 	return status;
       }
-    attribute_set_get_flags (attribute, amd_get_attr_flags, msg);
-    attribute_set_set_flags (attribute, amd_set_attr_flags, msg);
-    attribute_set_unset_flags (attribute, amd_unset_attr_flags, msg);
-    message_set_attribute (msg, attribute, mhm);
+    mu_attribute_set_get_flags (attribute, amd_get_attr_flags, msg);
+    mu_attribute_set_set_flags (attribute, amd_set_attr_flags, msg);
+    mu_attribute_set_unset_flags (attribute, amd_unset_attr_flags, msg);
+    mu_message_set_attribute (msg, attribute, mhm);
   }
 
   /* Prepare the body.  */
   {
-    body_t body = NULL;
-    stream_t stream = NULL;
-    if ((status = body_create (&body, msg)) != 0
-	|| (status = stream_create (&stream,
+    mu_body_t body = NULL;
+    mu_stream_t stream = NULL;
+    if ((status = mu_body_create (&body, msg)) != 0
+	|| (status = mu_stream_create (&stream,
 				    mailbox->flags | MU_STREAM_SEEKABLE,
 				    body)) != 0)
       {
-	body_destroy (&body, msg);
-	stream_destroy (&stream, body);
-	message_destroy (&msg, mhm);
+	mu_body_destroy (&body, msg);
+	mu_stream_destroy (&stream, body);
+	mu_message_destroy (&msg, mhm);
 	return status;
       }
-    stream_set_read (stream, amd_body_read, body);
-    stream_set_readline (stream, amd_body_readline, body);
-    stream_set_size (stream, amd_stream_size, body);
-    body_set_stream (body, stream, msg);
-    body_set_size (body, amd_body_size, msg);
-    body_set_lines (body, amd_body_lines, msg);
-    message_set_body (msg, body, mhm);
+    mu_stream_set_read (stream, amd_body_read, body);
+    mu_stream_set_readline (stream, amd_body_readline, body);
+    mu_stream_set_size (stream, amd_stream_size, body);
+    mu_body_set_stream (body, stream, msg);
+    mu_body_set_size (body, amd_body_size, msg);
+    mu_body_set_lines (body, amd_body_lines, msg);
+    mu_message_set_body (msg, body, mhm);
   }
 
   /* Set the envelope.  */
   {
-    envelope_t envelope = NULL;
-    status = envelope_create (&envelope, msg);
+    mu_envelope_t envelope = NULL;
+    status = mu_envelope_create (&envelope, msg);
     if (status != 0)
       {
-	message_destroy (&msg, mhm);
+	mu_message_destroy (&msg, mhm);
 	return status;
       }
-    envelope_set_sender (envelope, amd_envelope_sender, msg);
-    envelope_set_date (envelope, amd_envelope_date, msg);
-    message_set_envelope (msg, envelope, mhm);
+    mu_envelope_set_sender (envelope, amd_envelope_sender, msg);
+    mu_envelope_set_date (envelope, amd_envelope_date, msg);
+    mu_message_set_envelope (msg, envelope, mhm);
   }
 
   /* Set the UID.  */
   if (mhm->amd->message_uid)
-    message_set_uid (msg, mhm->amd->message_uid, mhm);
+    mu_message_set_uid (msg, mhm->amd->message_uid, mhm);
 
   /* Attach the message to the mailbox mbox data.  */
   mhm->message = msg;
-  message_set_mailbox (msg, mailbox, mhm);
+  mu_message_set_mailbox (msg, mailbox, mhm);
 
   if (pmsg)
     *pmsg = msg;
@@ -483,7 +484,7 @@ _amd_attach_message (mailbox_t mailbox, struct _amd_message *mhm,
 }
 
 static int
-amd_get_message (mailbox_t mailbox, size_t msgno, message_t *pmsg)
+amd_get_message (mu_mailbox_t mailbox, size_t msgno, mu_message_t *pmsg)
 {
   int status;
   struct _amd_data *amd = mailbox->data;
@@ -533,27 +534,29 @@ _amd_delim (char *str)
 static int
 _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
 {
-  stream_t stream = NULL;
+  mu_stream_t stream = NULL;
   char *name = NULL, *buf = NULL, *msg_name;
   size_t n, off = 0;
   size_t bsize;
   size_t nlines, nbytes;
   size_t new_body_start, new_header_lines;
   FILE *fp;
-  message_t msg = mhm->message;
-  header_t hdr;
+  mu_message_t msg = mhm->message;
+  mu_header_t hdr;
   int status;
-  attribute_t attr;
-  body_t body;
+  mu_attribute_t attr;
+  mu_body_t body;
   char buffer[512];
-  envelope_t env = NULL;
+  mu_envelope_t env = NULL;
   
+  status = mu_message_size (msg, &bsize);
+  if (status)
+    return status;
+
   fp = _amd_tempfile (mhm->amd, &name);
   if (!fp)
     return errno;
 
-  message_size (msg, &bsize);
-
   /* Try to allocate large buffer */
   for (; bsize > 1; bsize /= 2)
     if ((buf = malloc (bsize)))
@@ -563,11 +566,11 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
     return ENOMEM;
 
   /* Copy flags */
-  message_get_header (msg, &hdr);
-  header_get_stream (hdr, &stream);
+  mu_message_get_header (msg, &hdr);
+  mu_header_get_stream (hdr, &stream);
   off = 0;
   nlines = nbytes = 0;
-  while ((status = stream_readline (stream, buf, bsize, off, &n)) == 0
+  while ((status = mu_stream_readline (stream, buf, bsize, off, &n)) == 0
 	 && n != 0)
     {
       if (_amd_delim(buf))
@@ -596,8 +599,8 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
       nlines++;
     }
   
-  message_get_envelope (msg, &env);
-  if (envelope_date (env, buffer, sizeof buffer, &n) == 0 && n > 0)
+  mu_message_get_envelope (msg, &env);
+  if (mu_envelope_date (env, buffer, sizeof buffer, &n) == 0 && n > 0)
     {
       /* NOTE: buffer is terminated with \n */
       char *p = buffer;
@@ -611,15 +614,15 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
       nlines++;
     }
 	  
-  if (envelope_sender (env, buffer, sizeof buffer, &n) == 0 && n > 0)
+  if (mu_envelope_sender (env, buffer, sizeof buffer, &n) == 0 && n > 0)
     {
       fprintf (fp, "%s: %s\n", MU_HEADER_ENV_SENDER, buffer);
       nlines++;
     }
   
   /* Add status */
-  message_get_attribute (msg, &attr);
-  attribute_to_string (attr, buf, bsize, &n);
+  mu_message_get_attribute (msg, &attr);
+  mu_attribute_to_string (attr, buf, bsize, &n);
   if (n)
     {
       nbytes += fprintf (fp, "%s", buf);
@@ -633,11 +636,11 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
 
   /* Copy message body */
 
-  message_get_body (msg, &body);
-  body_get_stream (body, &stream);
+  mu_message_get_body (msg, &body);
+  mu_body_get_stream (body, &stream);
   off = 0;
   nlines = 0;
-  while (stream_read (stream, buf, bsize, off, &n) == 0 && n != 0)
+  while (mu_stream_read (stream, buf, bsize, off, &n) == 0 && n != 0)
     {
       char *p;
       for (p = buf; p < buf + n; p++)
@@ -665,7 +668,7 @@ _amd_message_save (struct _amd_data *amd, struct _amd_message *mhm, int expunge)
 }
 
 static int
-amd_append_message (mailbox_t mailbox, message_t msg)
+amd_append_message (mu_mailbox_t mailbox, mu_message_t msg)
 {
   int status;
   struct _amd_data *amd = mailbox->data;
@@ -720,7 +723,7 @@ amd_append_message (mailbox_t mailbox, message_t msg)
 }
 
 static int
-amd_messages_count (mailbox_t mailbox, size_t *pcount)
+amd_messages_count (mu_mailbox_t mailbox, size_t *pcount)
 {
   struct _amd_data *amd = mailbox->data;
 
@@ -740,7 +743,7 @@ amd_messages_count (mailbox_t mailbox, size_t *pcount)
    ('O' in the Status header), i.e. a message that is first seen
    by the current session (see attributes.h) */
 static int
-amd_messages_recent (mailbox_t mailbox, size_t *pcount)
+amd_messages_recent (mu_mailbox_t mailbox, size_t *pcount)
 {
   struct _amd_data *amd = mailbox->data;
   size_t count, i;
@@ -764,7 +767,7 @@ amd_messages_recent (mailbox_t mailbox, size_t *pcount)
 
 /* An "unseen" message is the one that has not been read yet */
 static int
-amd_message_unseen (mailbox_t mailbox, size_t *pmsgno)
+amd_message_unseen (mu_mailbox_t mailbox, size_t *pmsgno)
 {
   struct _amd_data *amd = mailbox->data;
   size_t i;
@@ -789,7 +792,7 @@ amd_message_unseen (mailbox_t mailbox, size_t *pmsgno)
 }
 
 static int
-amd_expunge (mailbox_t mailbox)
+amd_expunge (mu_mailbox_t mailbox)
 {
   struct _amd_data *amd = mailbox->data;
   struct _amd_message *mhm;
@@ -807,7 +810,7 @@ amd_expunge (mailbox_t mailbox)
       mhm = amd->msg_array[i];
       if ((mhm->attr_flags & MU_ATTRIBUTE_MODIFIED) ||
 	  (mhm->attr_flags & MU_ATTRIBUTE_DELETED) ||
-	  (mhm->message && message_is_modified (mhm->message)))
+	  (mhm->message && mu_message_is_modified (mhm->message)))
 	break;
     }
 
@@ -819,13 +822,18 @@ amd_expunge (mailbox_t mailbox)
 	{
 	  if (!mhm->deleted)
 	    {
-	      char *old_name, *new_name;
-	      /* Rename original message */
-	      old_name = amd->msg_file_name (mhm, 0);
-	      new_name = amd->msg_file_name (mhm, 1);
-	      rename (old_name, new_name);
+	      char *old_name = amd->msg_file_name (mhm, 0);
+	      char *new_name = amd->msg_file_name (mhm, 1);
+	      if (new_name)
+		{
+		  /* Rename original message */
+		  rename (old_name, new_name);
+		  free (new_name);
+		}
+	      else
+		/* Unlink original file */
+		unlink (old_name);
 	      free (old_name);
-	      free (new_name);
 	    }
 	  _amd_message_delete (amd, mhm);
 	  /* Do not increase i! */
@@ -833,7 +841,7 @@ amd_expunge (mailbox_t mailbox)
       else
 	{
 	  if ((mhm->attr_flags & MU_ATTRIBUTE_MODIFIED)
-	      || (mhm->message && message_is_modified (mhm->message)))
+	      || (mhm->message && mu_message_is_modified (mhm->message)))
 	    {
 	      _amd_attach_message (mailbox, mhm, NULL);
 	      mhm->deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
@@ -847,7 +855,7 @@ amd_expunge (mailbox_t mailbox)
 }
 
 static int
-amd_save_attributes (mailbox_t mailbox)
+amd_save_attributes (mu_mailbox_t mailbox)
 {
   struct _amd_data *amd = mailbox->data;
   struct _amd_message *mhm;
@@ -864,7 +872,7 @@ amd_save_attributes (mailbox_t mailbox)
     {
       mhm = amd->msg_array[i];
       if ((mhm->attr_flags & MU_ATTRIBUTE_MODIFIED)
-	  || (mhm->message && message_is_modified (mhm->message)))
+	  || (mhm->message && mu_message_is_modified (mhm->message)))
 	break;
     }
 
@@ -873,7 +881,7 @@ amd_save_attributes (mailbox_t mailbox)
       mhm = amd->msg_array[i];
 
       if ((mhm->attr_flags & MU_ATTRIBUTE_MODIFIED)
-	  || (mhm->message && message_is_modified (mhm->message)))
+	  || (mhm->message && mu_message_is_modified (mhm->message)))
 	{
 	  _amd_attach_message (mailbox, mhm, NULL);
 	  mhm->deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
@@ -885,7 +893,7 @@ amd_save_attributes (mailbox_t mailbox)
 }
 
 static int
-amd_uidvalidity (mailbox_t mailbox, unsigned long *puidvalidity)
+amd_uidvalidity (mu_mailbox_t mailbox, unsigned long *puidvalidity)
 {
   struct _amd_data *amd = mailbox->data;
   int status = amd_messages_count (mailbox, NULL);
@@ -904,14 +912,14 @@ amd_uidvalidity (mailbox_t mailbox, unsigned long *puidvalidity)
 }
 
 static int
-amd_uidnext (mailbox_t mailbox, size_t *puidnext)
+amd_uidnext (mu_mailbox_t mailbox, size_t *puidnext)
 {
   struct _amd_data *amd = mailbox->data;
   int status;
   
   if (!amd->next_uid)
     return ENOSYS;
-  status = mailbox_messages_count (mailbox, NULL);
+  status = mu_mailbox_messages_count (mailbox, NULL);
   if (status != 0)
     return status;
   /* If we did not start a scanning yet do it now.  */
@@ -930,9 +938,9 @@ amd_uidnext (mailbox_t mailbox, size_t *puidnext)
 void
 amd_cleanup (void *arg)
 {
-  mailbox_t mailbox = arg;
-  monitor_unlock (mailbox->monitor);
-  locker_unlock (mailbox->locker);
+  mu_mailbox_t mailbox = arg;
+  mu_monitor_unlock (mailbox->monitor);
+  mu_locker_unlock (mailbox->locker);
 }
 
 /* Insert message msg into the message list on the appropriate position */
@@ -977,7 +985,7 @@ _amd_message_delete (struct _amd_data *amd, struct _amd_message *msg)
   if (pp)
     *pp = NULL;
   
-  message_destroy (&msg->message, msg);
+  mu_message_destroy (&msg->message, msg);
   if (amd->msg_free)
     amd->msg_free (msg);
   free (msg);
@@ -989,9 +997,9 @@ _amd_message_delete (struct _amd_data *amd, struct _amd_message *msg)
 static int
 amd_scan_message (struct _amd_message *mhm)
 {
-  stream_t stream = mhm->stream;
+  mu_stream_t stream = mhm->stream;
   char buf[1024];
-  off_t off = 0;
+  mu_off_t off = 0;
   size_t n;
   int status;
   int in_header = 1;
@@ -1014,7 +1022,7 @@ amd_scan_message (struct _amd_message *mhm)
       free (msg_name);
     }
 
-  while ((status = stream_readline (stream, buf, sizeof (buf), off, &n) == 0)
+  while ((status = mu_stream_readline (stream, buf, sizeof (buf), off, &n) == 0)
 	 && n != 0)
     {
       if (in_header)
@@ -1031,7 +1039,7 @@ amd_scan_message (struct _amd_message *mhm)
 	  if (strncasecmp (buf, "status:", 7) == 0)
 	    {
 	      int deleted = mhm->attr_flags & MU_ATTRIBUTE_DELETED;
-	      string_to_flags (buf, &mhm->attr_flags);
+	      mu_string_to_flags (buf, &mhm->attr_flags);
 	      mhm->attr_flags |= deleted;
 	    }
 	  else if (strncasecmp (buf, "x-imapbase:", 11) == 0)
@@ -1059,7 +1067,7 @@ amd_scan_message (struct _amd_message *mhm)
 }
 
 static int
-amd_scan (mailbox_t mailbox, size_t msgno, size_t *pcount)
+amd_scan (mu_mailbox_t mailbox, size_t msgno, size_t *pcount)
 {
   struct _amd_data *amd = mailbox->data;
 
@@ -1075,7 +1083,7 @@ amd_scan (mailbox_t mailbox, size_t msgno, size_t *pcount)
 /* Is the internal representation of the mailbox up to date.
    Return 1 if so, 0 otherwise. */
 static int
-amd_is_updated (mailbox_t mailbox)
+amd_is_updated (mu_mailbox_t mailbox)
 {
   struct stat st;
   struct _amd_data *amd = mailbox->data;
@@ -1090,7 +1098,7 @@ amd_is_updated (mailbox_t mailbox)
 }
 
 static int
-amd_get_size (mailbox_t mailbox ARG_UNUSED, off_t *psize ARG_UNUSED)
+amd_get_size (mu_mailbox_t mailbox ARG_UNUSED, mu_off_t *psize ARG_UNUSED)
 {
   /*FIXME*/
   return ENOSYS;
@@ -1181,17 +1189,17 @@ amd_message_stream_open (struct _amd_message *mhm)
     flags |= MU_STREAM_RDWR;
   else 
     flags |= MU_STREAM_READ;
-  status = file_stream_create (&mhm->stream, filename, flags);
+  status = mu_file_stream_create (&mhm->stream, filename, flags);
 
   free (filename);
 
   if (status != 0)
     return status;
 
-  status = stream_open (mhm->stream);
+  status = mu_stream_open (mhm->stream);
 
   if (status != 0)
-    stream_destroy (&mhm->stream, NULL);
+    mu_stream_destroy (&mhm->stream, NULL);
 
   if (status == 0)
     status = amd_scan_message (mhm);
@@ -1205,7 +1213,7 @@ amd_message_stream_close (struct _amd_message *mhm)
 {
   if (mhm)
     {
-      stream_close (mhm->stream);
+      mu_stream_close (mhm->stream);
       mhm->stream = NULL;
     }
 }
@@ -1221,12 +1229,12 @@ amd_check_message (struct _amd_message *mhm)
 
 static int
 amd_readstream (struct _amd_message *mhm, char *buffer, size_t buflen,
-	       off_t off, size_t *pnread, int isreadline,
-	       off_t start, off_t end)
+	       mu_off_t off, size_t *pnread, int isreadline,
+	       mu_off_t start, mu_off_t end)
 {
   size_t nread = 0;
   int status = 0;
-  off_t ln;
+  mu_off_t ln;
 
   if (buffer == NULL || buflen == 0)
     {
@@ -1235,7 +1243,7 @@ amd_readstream (struct _amd_message *mhm, char *buffer, size_t buflen,
       return 0;
     }
 
-  monitor_rdlock (mhm->amd->mailbox->monitor);
+  mu_monitor_rdlock (mhm->amd->mailbox->monitor);
 #ifdef WITH_PTHREAD
   /* read() is cancellation point since we're doing a potentially
      long operation.  Lets make sure we clean the state.  */
@@ -1248,14 +1256,14 @@ amd_readstream (struct _amd_message *mhm, char *buffer, size_t buflen,
       /* Position the file pointer and the buffer.  */
       nread = ((size_t)ln < buflen) ? (size_t)ln : buflen;
       if (isreadline)
-	status = stream_readline (mhm->stream, buffer, buflen,
+	status = mu_stream_readline (mhm->stream, buffer, buflen,
 				  start + off, &nread);
       else
-	status = stream_read (mhm->stream, buffer, nread,
+	status = mu_stream_read (mhm->stream, buffer, nread,
 			      start + off, &nread);
     }
 
-  monitor_unlock (mhm->amd->mailbox->monitor);
+  mu_monitor_unlock (mhm->amd->mailbox->monitor);
 #ifdef WITH_PTHREAD
   pthread_cleanup_pop (0);
 #endif
@@ -1266,24 +1274,24 @@ amd_readstream (struct _amd_message *mhm, char *buffer, size_t buflen,
 }
 
 static int
-amd_body_read (stream_t is, char *buffer, size_t buflen, off_t off,
+amd_body_read (mu_stream_t is, char *buffer, size_t buflen, mu_off_t off,
 	      size_t *pnread)
 {
-  body_t body = stream_get_owner (is);
-  message_t msg = body_get_owner (body);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_body_t body = mu_stream_get_owner (is);
+  mu_message_t msg = mu_body_get_owner (body);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   amd_pool_open (mhm);
   return amd_readstream (mhm, buffer, buflen, off, pnread, 0,
 			mhm->body_start, mhm->body_end);
 }
 
 static int
-amd_body_readline (stream_t is, char *buffer, size_t buflen,
-		  off_t off, size_t *pnread)
+amd_body_readline (mu_stream_t is, char *buffer, size_t buflen,
+		  mu_off_t off, size_t *pnread)
 {
-  body_t body = stream_get_owner (is);
-  message_t msg = body_get_owner (body);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_body_t body = mu_stream_get_owner (is);
+  mu_message_t msg = mu_body_get_owner (body);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   amd_pool_open (mhm);
   return amd_readstream (mhm, buffer, buflen, off, pnread, 1,
 			mhm->body_start, mhm->body_end);
@@ -1292,17 +1300,17 @@ amd_body_readline (stream_t is, char *buffer, size_t buflen,
 /* Return corresponding sizes */
 
 static int
-amd_stream_size (stream_t stream, off_t *psize)
+amd_stream_size (mu_stream_t stream, mu_off_t *psize)
 {
-  body_t body = stream_get_owner (stream);
+  mu_body_t body = mu_stream_get_owner (stream);
   return amd_body_size (body, (size_t*) psize);
 }
 
 static int
-amd_body_size (body_t body, size_t *psize)
+amd_body_size (mu_body_t body, size_t *psize)
 {
-  message_t msg = body_get_owner (body);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_body_get_owner (body);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   if (mhm == NULL)
     return EINVAL;
   amd_check_message (mhm);
@@ -1312,10 +1320,10 @@ amd_body_size (body_t body, size_t *psize)
 }
 
 static int
-amd_body_lines (body_t body, size_t *plines)
+amd_body_lines (mu_body_t body, size_t *plines)
 {
-  message_t msg = body_get_owner (body);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_body_get_owner (body);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   if (mhm == NULL)
     return EINVAL;
   amd_check_message (mhm);
@@ -1326,11 +1334,11 @@ amd_body_lines (body_t body, size_t *plines)
 
 /* Headers */
 static int
-amd_header_fill (header_t header, char *buffer, size_t len,
-		off_t off, size_t *pnread)
+amd_header_fill (mu_header_t header, char *buffer, size_t len,
+		mu_off_t off, size_t *pnread)
 {
-  message_t msg = header_get_owner (header);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_header_get_owner (header);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
 
   amd_pool_open (mhm);
   return amd_readstream (mhm, buffer, len, off, pnread, 0,
@@ -1338,10 +1346,10 @@ amd_header_fill (header_t header, char *buffer, size_t len,
 }
 
 static int
-amd_header_size (header_t header, size_t *psize)
+amd_header_size (mu_header_t header, size_t *psize)
 {
-  message_t msg = header_get_owner (header);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_header_get_owner (header);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   if (mhm == NULL)
     return EINVAL;
   amd_check_message (mhm);
@@ -1351,10 +1359,10 @@ amd_header_size (header_t header, size_t *psize)
 }
 
 static int
-amd_header_lines (header_t header, size_t *plines)
+amd_header_lines (mu_header_t header, size_t *plines)
 {
-  message_t msg = header_get_owner (header);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_header_get_owner (header);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
   if (mhm == NULL)
     return EINVAL;
   amd_check_message (mhm);
@@ -1365,10 +1373,10 @@ amd_header_lines (header_t header, size_t *plines)
 
 /* Attributes */
 static int
-amd_get_attr_flags (attribute_t attr, int *pflags)
+amd_get_attr_flags (mu_attribute_t attr, int *pflags)
 {
-  message_t msg = attribute_get_owner (attr);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_attribute_get_owner (attr);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
 
   if (mhm == NULL)
     return EINVAL;
@@ -1378,10 +1386,10 @@ amd_get_attr_flags (attribute_t attr, int *pflags)
 }
 
 static int
-amd_set_attr_flags (attribute_t attr, int flags)
+amd_set_attr_flags (mu_attribute_t attr, int flags)
 {
-  message_t msg = attribute_get_owner (attr);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_attribute_get_owner (attr);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
 
   if (mhm == NULL)
     return EINVAL;
@@ -1390,10 +1398,10 @@ amd_set_attr_flags (attribute_t attr, int flags)
 }
 
 static int
-amd_unset_attr_flags (attribute_t attr, int flags)
+amd_unset_attr_flags (mu_attribute_t attr, int flags)
 {
-  message_t msg = attribute_get_owner (attr);
-  struct _amd_message *mhm = message_get_owner (msg);
+  mu_message_t msg = mu_attribute_get_owner (attr);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
 
   if (mhm == NULL)
     return EINVAL;
@@ -1403,60 +1411,74 @@ amd_unset_attr_flags (attribute_t attr, int flags)
 
 /* Envelope */
 static int
-amd_envelope_date (envelope_t envelope, char *buf, size_t len,
+amd_envelope_date (mu_envelope_t envelope, char *buf, size_t len,
 		  size_t *psize)
 {
-  message_t msg = envelope_get_owner (envelope);
-  struct _amd_message *mhm = message_get_owner (msg);
-  header_t hdr = NULL;
-  char *from;
+  mu_message_t msg = mu_envelope_get_owner (envelope);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
+  mu_header_t hdr = NULL;
+  char *date;
   int status;
   
   if (mhm == NULL)
     return EINVAL;
 
-  if ((status = message_get_header (msg, &hdr)) != 0)
+  if ((status = mu_message_get_header (msg, &hdr)) != 0)
     return status;
-  if (header_aget_value (hdr, MU_HEADER_ENV_DATE, &from)
-      && header_aget_value (hdr, MU_HEADER_DELIVERY_DATE, &from))
-    return ENOSYS;
+  if (mu_header_aget_value (hdr, MU_HEADER_ENV_DATE, &date)
+      && mu_header_aget_value (hdr, MU_HEADER_DELIVERY_DATE, &date))
+    return MU_ERR_NOENT;
+  else
+    {
+      time_t t;
+      int rc;
+      
+      /* Convert to ctime format */
+      rc = mu_parse_date (date, &t, NULL); /* FIXME: TZ info is lost */
+      free (date);
+      if (rc)
+	return MU_ERR_NOENT;
+      date = strdup (ctime (&t)); 
+    }
 
   /* Format:  "sender date" */
   if (buf && len > 0)
     {
       len--; /* Leave space for the null.  */
-      strncpy (buf, from, len);
-      if (strlen (from) < len)
+      strncpy (buf, date, len);
+      if (strlen (date) < len)
 	{
 	  len = strlen (buf);
-	  buf[len++] = '\n';
+	  if (buf[len-1] != '\n')
+	    buf[len++] = '\n';
 	}
       buf[len] = '\0';
     }
   else
     len = 0;
-
+  free (date);
+  
   if (psize)
     *psize = len;
   return 0;
 }
 
 static int
-amd_envelope_sender (envelope_t envelope, char *buf, size_t len, size_t *psize)
+amd_envelope_sender (mu_envelope_t envelope, char *buf, size_t len, size_t *psize)
 {
-  message_t msg = envelope_get_owner (envelope);
-  struct _amd_message *mhm = message_get_owner (msg);
-  header_t hdr = NULL;
+  mu_message_t msg = mu_envelope_get_owner (envelope);
+  struct _amd_message *mhm = mu_message_get_owner (msg);
+  mu_header_t hdr = NULL;
   char *from;
   int status;
 
   if (mhm == NULL)
     return EINVAL;
 
-  if ((status = message_get_header (msg, &hdr)) != 0)
+  if ((status = mu_message_get_header (msg, &hdr)) != 0)
+    return status;
+  if (status = mu_header_aget_value (hdr, MU_HEADER_ENV_SENDER, &from))
     return status;
-  if (header_aget_value (hdr, MU_HEADER_ENV_SENDER, &from))
-    return ENOSYS;
 
   if (buf && len > 0)
     {
@@ -1476,14 +1498,14 @@ amd_envelope_sender (envelope_t envelope, char *buf, size_t len, size_t *psize)
 }
 
 static void
-amd_url_destroy (url_t url ARG_UNUSED)
+amd_url_destroy (mu_url_t url ARG_UNUSED)
 {
 }
 
 int
-amd_url_init (url_t url, const char *scheme)
+amd_url_init (mu_url_t url, const char *scheme)
 {
-  const char *name = url_to_string (url);
+  const char *name = mu_url_to_string (url);
   const char *path_ptr = name;
   size_t len = strlen (name);
   size_t scheme_len = strlen (scheme);
