@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2004, 2005 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -11,9 +11,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   You should have received a copy of the GNU Lesser General
+   Public License along with this library; if not, write to the
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301 USA */
 
 /* Notes:
 First Draft: Dave Inglis.
@@ -55,12 +56,12 @@ struct _trans_stream
   int w_whd;       /* Working buffer write ahead */
   char w_buf[MU_TRANS_BSIZE]; /* working source/dest buffer */
 
-  int (*transcoder) __P ((const char *iptr, size_t isize, char *optr,
-			  size_t osize, size_t *nbytes, int *line_len));
+  int (*transcoder) (const char *iptr, size_t isize, char *optr,
+		     size_t osize, size_t *nbytes, int *line_len);
 };
 
 static void
-trans_destroy (filter_t filter)
+trans_destroy (mu_filter_t filter)
 {
   struct _trans_stream *ts = filter->data;
   if (ts->s_buf)
@@ -69,7 +70,7 @@ trans_destroy (filter_t filter)
 }
 
 static int
-trans_read (filter_t filter, char *optr, size_t osize, off_t offset,
+trans_read (mu_filter_t filter, char *optr, size_t osize, mu_off_t offset,
 	    size_t *n_bytes)
 {
   struct _trans_stream *ts = filter->data;
@@ -100,7 +101,7 @@ trans_read (filter_t filter, char *optr, size_t osize, off_t offset,
 	  memmove (ts->w_buf, ts->w_buf + ts->w_rhd, ts->w_whd - ts->w_rhd);
 	  ts->w_whd = ts->w_whd - ts->w_rhd;
 	  ts->w_rhd = 0;
-	  ret = stream_read (filter->stream, ts->w_buf + ts->w_whd,
+	  ret = mu_stream_read (filter->stream, ts->w_buf + ts->w_whd,
 			     MU_TRANS_BSIZE - ts->w_whd, ts->offset,
 			     &wbytes );
 	  if (ret != 0)
@@ -284,7 +285,6 @@ qp_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 {
   unsigned int c;
   size_t consumed = 0;
-  size_t wscount = 0;
 
   *nbytes = 0;
 
@@ -318,9 +318,6 @@ qp_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 	  *line_len = 0;
 	}
 	  
-      if (ISWS (c))
-	wscount++;
-	
       if (simple_char)
 	{
 	  /* a non-quoted character uses up one byte */
@@ -334,9 +331,6 @@ qp_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 	  iptr++;
 	  consumed++;
 
-	  if (!ISWS (c))
-	    wscount = 0;
-	  
 	  if (c == '\n')
 	    *line_len = 0;
 	}
@@ -356,17 +350,13 @@ qp_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 	  /* we've actuall used up one byte of input */
 	  iptr++;
 	  consumed++;
-	  wscount = 0;
 	}
     }
-  if (wscount)
-    wscount--;
-  (*nbytes) -= wscount;
-  return consumed - wscount;
+  return consumed;
 }
 
 static int
-qp_init (filter_t filter)
+qp_init (mu_filter_t filter)
 {
   struct _trans_stream *ts;
   ts = calloc (sizeof (*ts), 1);
@@ -388,7 +378,7 @@ qp_init (filter_t filter)
   return 0;
 }
 
-static struct _filter_record _qp_filter =
+static struct _mu_filter_record _qp_filter =
 {
   "quoted-printable",
   qp_init,
@@ -498,7 +488,7 @@ base64_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 }
 
 static int
-base64_init (filter_t filter)
+base64_init (mu_filter_t filter)
 {
   struct _trans_stream *ts;
   ts = calloc (sizeof (*ts), 1);
@@ -520,7 +510,7 @@ base64_init (filter_t filter)
   return 0;
 }
 
-static struct _filter_record _base64_filter =
+static struct _mu_filter_record _base64_filter =
 {
   "base64",
   base64_init,
@@ -529,6 +519,15 @@ static struct _filter_record _base64_filter =
   NULL
 };
 
+static struct _mu_filter_record _B_filter =
+{
+  "B",
+  base64_init,
+  NULL,
+  NULL,
+  NULL
+};
+
 
 
 /* RFC 2047 "Q" Encoding */
@@ -618,9 +617,10 @@ Q_printable_char_p (unsigned c)
     case '?':
     case '_':
     case ' ':
+    case '\t':
       return 0;
     default:
-      return 1;
+      return c > 32 && c < 127;
     }
 }
 
@@ -649,9 +649,13 @@ Q_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 	  iptr++;
 	  consumed++;
 	}
-      else if (c == '_')
+      else if (c == 0x20)
 	{
-	  *optr++ = ' ';
+	  /* RFC2047, 4.2.2:
+	     Note that the "_"
+	     always represents hexadecimal 20, even if the SPACE character
+	     occupies a different code position in the character set in use. */
+	  *optr++ = '_';
 	  (*nbytes)++;
 	  (*line_len)++;
 	  iptr++;
@@ -679,7 +683,7 @@ Q_encode (const char *iptr, size_t isize, char *optr, size_t osize,
 }
 
 static int
-Q_init (filter_t filter)
+Q_init (mu_filter_t filter)
 {
   struct _trans_stream *ts;
   ts = calloc (sizeof (*ts), 1);
@@ -701,7 +705,7 @@ Q_init (filter_t filter)
   return 0;
 }
 
-static struct _filter_record _Q_filter =
+static struct _mu_filter_record _Q_filter =
 {
   "Q",
   Q_init,
@@ -713,7 +717,7 @@ static struct _filter_record _Q_filter =
 
 /* Pass-through encodings */
 
-static struct _filter_record _bit8_filter =
+static struct _mu_filter_record _bit8_filter =
 {
   "8bit",
   NULL,
@@ -722,7 +726,7 @@ static struct _filter_record _bit8_filter =
   NULL
 };
 
-static struct _filter_record _bit7_filter =
+static struct _mu_filter_record _bit7_filter =
 {
   "7bit",
   NULL,
@@ -731,7 +735,7 @@ static struct _filter_record _bit7_filter =
   NULL
 };
 
-static struct _filter_record _binary_filter =
+static struct _mu_filter_record _binary_filter =
 {
   "binary",
   NULL,
@@ -743,13 +747,13 @@ static struct _filter_record _binary_filter =
 
 
 /* Export.  */
-filter_record_t qp_filter = &_qp_filter;
-filter_record_t base64_filter = &_base64_filter;
-filter_record_t binary_filter = &_binary_filter;
-filter_record_t bit8_filter = &_bit8_filter;
-filter_record_t bit7_filter = &_bit7_filter;
-filter_record_t rfc_2047_Q_filter = &_Q_filter;
-
+mu_filter_record_t mu_qp_filter = &_qp_filter;
+mu_filter_record_t mu_base64_filter = &_base64_filter;
+mu_filter_record_t mu_binary_filter = &_binary_filter;
+mu_filter_record_t mu_bit8_filter = &_bit8_filter;
+mu_filter_record_t mu_bit7_filter = &_bit7_filter;
+mu_filter_record_t mu_rfc_2047_Q_filter = &_Q_filter;
+mu_filter_record_t mu_rfc_2047_B_filter = &_B_filter;
 
 
 
