@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
    Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2005 Free Software Foundation, Inc.
+   2005, 2006 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -36,6 +36,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/select.h>
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -102,7 +103,7 @@ mu_get_homedir (void)
     homedir = strdup (homedir);
   else
     {
-      struct mu_auth_data *auth = mu_get_auth_by_uid (getuid ());
+      struct mu_auth_data *auth = mu_get_auth_by_uid (geteuid ());
       if (!auth)
 	return NULL;
       homedir = strdup (auth->dir);
@@ -183,9 +184,33 @@ mu_get_full_path (const char *file)
 char *
 mu_tilde_expansion (const char *ref, const char *delim, const char *homedir)
 {
-  char *p = strdup (ref);
+  char *base = strdup (ref);
   char *home = NULL;
+  char *proto = NULL;
+  size_t proto_len = 0;
+  char *p;
 
+  for (p = base; *p && isascii (*p) && isalnum (*p); p++)
+    ;
+  
+  if (*p == ':')
+    {
+      p++;
+      proto_len = p - base;
+      proto = malloc (proto_len + 1);
+      if (!proto)
+	return NULL;
+      memcpy (proto, base, proto_len);
+      proto[proto_len] = 0;
+      /* Allow for extra pair of slashes after the protocol specifier */
+      if (*p == delim[0])
+	p++;
+      if (*p == delim[0])
+	p++;
+    }
+  else
+    p = base;
+  
   if (*p == '~')
     {
       p++;
@@ -196,14 +221,18 @@ mu_tilde_expansion (const char *ref, const char *delim, const char *homedir)
 	    {
 	      home = mu_get_homedir ();
 	      if (!home)
-		return NULL;
+		return base;
 	      homedir = home;
 	    }
-	  s = calloc (strlen (homedir) + strlen (p) + 1, 1);
-          strcpy (s, homedir);
+	  s = calloc (proto_len + strlen (homedir) + strlen (p) + 1, 1);
+	  if (proto_len)
+	    strcpy (s, proto);
+	  else
+	    s[0] = 0;
+          strcat (s, homedir);
           strcat (s, p);
-          free (--p);
-          p = s;
+          free (base);
+          base = s;
         }
       else
         {
@@ -214,26 +243,29 @@ mu_tilde_expansion (const char *ref, const char *delim, const char *homedir)
             s++;
           name = calloc (s - p + 1, 1);
           memcpy (name, p, s - p);
-          name [s - p] = '\0';
+          name[s - p] = '\0';
 	  
           auth = mu_get_auth_by_name (name);
           free (name);
           if (auth)
             {
-              char *buf = calloc (strlen (auth->dir) + strlen (s) + 1, 1);
-	      strcpy (buf, auth->dir);
+              char *buf = calloc (proto_len + strlen (auth->dir)
+				  + strlen (s) + 1, 1);
+	      if (proto_len)
+		strcpy (buf, proto);
+	      else
+		buf[0] = 0;
+	      strcat (buf, auth->dir);
               strcat (buf, s);
-              free (--p);
-              p = buf;
+              free (base);
+              base = buf;
 	      mu_auth_data_free (auth);
             }
-          else
-            p--;
         }
     }
   if (home)
     free (home);
-  return p;
+  return base;
 }
 
 /* Smart strncpy that always add the null and returns the number of bytes
@@ -254,30 +286,30 @@ mu_cpystr (char *dst, const char *src, size_t size)
 /* General retrieve stack support: */
 
 void
-mu_register_retriever (list_t *pflist, mu_retrieve_fp fun)
+mu_register_retriever (mu_list_t *pflist, mu_retrieve_fp fun)
 {
-  if (!*pflist && list_create (pflist))
+  if (!*pflist && mu_list_create (pflist))
     return;
-  list_append (*pflist, fun);
+  mu_list_append (*pflist, fun);
 }
 
 void *
-mu_retrieve (list_t flist, void *data)
+mu_retrieve (mu_list_t flist, void *data)
 {
   void *p = NULL;
-  iterator_t itr;
+  mu_iterator_t itr;
 
-  if (list_get_iterator (flist, &itr) == 0)
+  if (mu_list_get_iterator (flist, &itr) == 0)
     {
       mu_retrieve_fp fun;
-      for (iterator_first (itr); !p && !iterator_is_done (itr);
-	   iterator_next (itr))
+      for (mu_iterator_first (itr); !p && !mu_iterator_is_done (itr);
+	   mu_iterator_next (itr))
 	{
-	  iterator_current (itr, (void **)&fun);
+	  mu_iterator_current (itr, (void **)&fun);
 	  p = (*fun) (data);
 	}
 
-      iterator_destroy (&itr);
+      mu_iterator_destroy (&itr);
     }
   return p;
 }
@@ -317,15 +349,15 @@ int
 mu_set_user_email (const char *candidate)
 {
   int err = 0;
-  address_t addr = NULL;
+  mu_address_t addr = NULL;
   size_t emailno = 0;
   char *email = NULL;
   char *domain = NULL;
   
-  if ((err = address_create (&addr, candidate)) != 0)
+  if ((err = mu_address_create (&addr, candidate)) != 0)
     return err;
 
-  if ((err = address_get_email_count (addr, &emailno)) != 0)
+  if ((err = mu_address_get_email_count (addr, &emailno)) != 0)
     goto cleanup;
 
   if (emailno != 1)
@@ -334,7 +366,7 @@ mu_set_user_email (const char *candidate)
       goto cleanup;
     }
 
-  if ((err = address_aget_email (addr, 1, &email)) != 0)
+  if ((err = mu_address_aget_email (addr, 1, &email)) != 0)
     goto cleanup;
 
   if (mu_user_email)
@@ -342,12 +374,12 @@ mu_set_user_email (const char *candidate)
 
   mu_user_email = email;
 
-  address_aget_domain (addr, 1, &domain);
+  mu_address_aget_domain (addr, 1, &domain);
   mu_set_user_email_domain (domain);
   free (domain);
   
 cleanup:
-  address_destroy (&addr);
+  mu_address_destroy (&addr);
 
   return err;
 }
@@ -431,7 +463,7 @@ mu_get_user_email (const char *name)
       return NULL;
     }
 
-  if ((status = parse822_quote_local_part (&localpart, name)))
+  if ((status = mu_parse822_quote_local_part (&localpart, name)))
     {
       free(tmpname);
       errno = status;
@@ -517,25 +549,33 @@ mu_normalize_path (char *path, const char *delim)
   return path;
 }
 
-char *
-mu_normalize_maildir (const char *dir)
+int
+mu_normalize_mailbox_url (char **pout, const char *dir)
 {
-  int len = strlen (dir);
+  int len;
+
+  if (!pout)
+    return MU_ERR_OUT_PTR_NULL;
+      
+  len = strlen (dir);
   if (dir[len-1] == '/')
-    return strdup (dir);
+    *pout = strdup (dir);
   else if (strncasecmp (dir, "mbox:", 5) == 0 && dir[len-1] == '=')
     {
       if (len > 5 && strcmp (dir + len - 5, "user=") == 0)
-	return strdup (dir);
+	*pout = strdup (dir);
       else
-	return NULL;
+	return MU_ERR_BAD_FILENAME;
     }
   else
     {
-      char *p = malloc (strlen (dir) + 2);
-      strcat (strcpy (p, dir), "/");
-      return p;
+      *pout = malloc (strlen (dir) + 2);
+      if (*pout)
+	strcat (strcpy (*pout, dir), "/");
     }
+
+  /* Final check */
+  return (*pout == NULL) ? errno : 0;
 }
 
 /* Create and open a temporary file. Be very careful about it, since we
@@ -914,10 +954,10 @@ strip_message_id (char *msgid, char **pval)
 }
 
 int
-get_msgid_header (header_t hdr, const char *name, char **val)
+get_msgid_header (mu_header_t hdr, const char *name, char **val)
 {
   char *p;
-  int status = header_aget_value (hdr, name, &p);
+  int status = mu_header_aget_value (hdr, name, &p);
   if (status)
     return status;
   status = strip_message_id (p, val);
@@ -960,13 +1000,13 @@ concat (const char *s1, const char *s2)
    References:" field. */
 
 int
-mu_rfc2822_references (message_t msg, char **pstr)
+mu_rfc2822_references (mu_message_t msg, char **pstr)
 {
   char *ref = NULL, *msgid = NULL;
-  header_t hdr;
+  mu_header_t hdr;
   int rc;
   
-  rc = message_get_header (msg, &hdr);
+  rc = mu_message_get_header (msg, &hdr);
   if (rc)
     return rc;
   get_msgid_header (hdr, MU_HEADER_MESSAGE_ID, &msgid);
@@ -996,24 +1036,24 @@ mu_rfc2822_references (message_t msg, char **pstr)
    field.
 */
 int
-mu_rfc2822_in_reply_to (message_t msg, char **pstr)
+mu_rfc2822_in_reply_to (mu_message_t msg, char **pstr)
 {
   char *value, *s1 = NULL, *s2 = NULL;
-  header_t hdr;
+  mu_header_t hdr;
   int rc;
   
-  rc = message_get_header (msg, &hdr);
+  rc = mu_message_get_header (msg, &hdr);
   if (rc)
     return rc;
   
-  if (header_aget_value (hdr, MU_HEADER_DATE, &value))
+  if (mu_header_aget_value (hdr, MU_HEADER_DATE, &value))
     {
-      envelope_t envelope = NULL;
+      mu_envelope_t envelope = NULL;
       value = malloc (DATEBUFSIZE);
       if (value)
 	{
-	  message_get_envelope (msg, &envelope);
-	  envelope_date (envelope, value, DATEBUFSIZE, NULL);
+	  mu_message_get_envelope (msg, &envelope);
+	  mu_envelope_date (envelope, value, DATEBUFSIZE, NULL);
 	}
     }
 
@@ -1027,7 +1067,7 @@ mu_rfc2822_in_reply_to (message_t msg, char **pstr)
 	return ENOMEM;
     }
   
-  if (header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &value) == 0)
+  if (mu_header_aget_value (hdr, MU_HEADER_MESSAGE_ID, &value) == 0)
     {
       s2 = malloc (strlen (value) + 3);
       if (s2)
@@ -1250,33 +1290,33 @@ mu_set_default_fallback (const char *str)
 }
 
 int
-mu_decode_filter (stream_t *pfilter, stream_t input,
+mu_decode_filter (mu_stream_t *pfilter, mu_stream_t input,
 		  const char *filter_type,
 		  const char *fromcode, const char *tocode)
 {
-  stream_t filter;
+  mu_stream_t filter;
   
-  int status = filter_create (&filter, input, filter_type,
+  int status = mu_filter_create (&filter, input, filter_type,
 			      MU_FILTER_DECODE, MU_STREAM_READ);
   if (status)
     return status;
 
   if (fromcode && tocode && strcasecmp (fromcode, tocode))
     {
-      stream_t cvt;
-      status = filter_iconv_create (&cvt, filter, fromcode, tocode,
+      mu_stream_t cvt;
+      status = mu_filter_iconv_create (&cvt, filter, fromcode, tocode,
 				    MU_STREAM_NO_CLOSE,
 				    mu_default_fallback_mode);
       if (status == 0)
 	{
-	  if (stream_open (cvt))
-	    stream_destroy (&cvt, stream_get_owner (cvt));
+	  if (mu_stream_open (cvt))
+	    mu_stream_destroy (&cvt, mu_stream_get_owner (cvt));
 	  else
 	    {
 	      int flags;
-	      stream_get_flags (cvt, &flags);
+	      mu_stream_get_flags (cvt, &flags);
 	      flags &= ~MU_STREAM_NO_CLOSE;
-	      stream_set_flags (cvt, flags);
+	      mu_stream_set_flags (cvt, flags);
 	      filter = cvt;
 	    }
 	}
@@ -1284,3 +1324,34 @@ mu_decode_filter (stream_t *pfilter, stream_t input,
   *pfilter = filter;
   return 0;
 }
+
+int
+mu_is_proto (const char *p)
+{
+  for (; *p && *p != '/'; p++)
+    if (*p == ':')
+      return 1;
+  return 0;
+}
+
+int
+mu_mh_delim (const char *str)
+{
+  if (str[0] == '-')
+    {
+      for (; *str == '-'; str++)
+	;
+      for (; *str == ' ' || *str == '\t'; str++)
+	;
+    }
+  return str[0] == '\n';
+}
+
+char *
+__argp_base_name (const char *arg)
+{
+  char *p = strrchr (arg, '/');
+  return p ? p + 1 : arg;
+}
+
+    
