@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 
+   2005 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,11 +14,13 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 /* Initialize MH applications. */
 
 #include <mh.h>
+#include <mailutils/url.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -167,12 +170,36 @@ mh_is_my_name (char *name)
       char *nlist = mh_global_profile_get ("Alternate-Mailboxes", NULL);
       if (nlist)
 	{
-	  char *p, *sp;
-      
-	  p = strtok_r (nlist, ",", &sp);
-	  do
-	    rc = emailcmp (p, pname) == 0;
-	  while (rc == 0 && (p = strtok_r (NULL, ",", &sp)));
+	  char *end, *p, *pat;
+	  int len;
+	  
+	  for (p = nlist; rc == 0 && *p; p = end)
+	    {
+	      
+	      while (*p && isspace (*p))
+		p++;
+
+	      end = strchr (p, ',');
+	      if (end)
+		{
+		  len = end - p;
+		  end++;
+		}
+	      else
+		{
+		  len = strlen (p);
+		  end = p + len;
+		}
+
+	      while (len > 0 && isspace (p[len-1]))
+		len--;
+
+	      pat = xmalloc (len + 1);
+	      memcpy (pat, p, len);
+	      pat[len] = 0;
+	      rc = emailcmp (pat, pname) == 0;
+	      free (pat);
+	    }
 	}
     }
   free (pname);
@@ -295,13 +322,13 @@ mh_getyn_interactive (const char *fmt, ...)
 }
 	    
 FILE *
-mh_audit_open (char *name, mailbox_t mbox)
+mh_audit_open (char *name, mu_mailbox_t mbox)
 {
   FILE *fp;
   char date[64];
   time_t t;
   struct tm *tm;
-  url_t url;
+  mu_url_t url;
   char *namep;
   
   namep = mu_tilde_expansion (name, "/", NULL);
@@ -309,7 +336,7 @@ mh_audit_open (char *name, mailbox_t mbox)
     {
       char *p = NULL;
 
-      asprintf (&p, "%s/%s", mu_path_folder_dir, namep);
+      asprintf (&p, "%s/%s", mu_folder_directory (), namep);
       if (!p)
 	{
 	  mh_error (_("Not enough memory"));
@@ -331,12 +358,12 @@ mh_audit_open (char *name, mailbox_t mbox)
   time (&t);
   tm = localtime (&t);
   strftime (date, sizeof date, "%a, %d %b %Y %H:%M:%S %Z", tm);
-  mailbox_get_url (mbox, &url);
+  mu_mailbox_get_url (mbox, &url);
   
   fprintf (fp, "<<%s>> %s %s\n",
 	   program_invocation_short_name,
 	   date,
-	   url_to_string (url));
+	   mu_url_to_string (url));
   return fp;
 }
 
@@ -347,15 +374,15 @@ mh_audit_close (FILE *fp)
 }
 
 int
-mh_message_number (message_t msg, size_t *pnum)
+mh_message_number (mu_message_t msg, size_t *pnum)
 {
-  return message_get_uid (msg, pnum);	
+  return mu_message_get_uid (msg, pnum);	
 }
 
-mailbox_t
+mu_mailbox_t
 mh_open_folder (const char *folder, int create)
 {
-  mailbox_t mbox = NULL;
+  mu_mailbox_t mbox = NULL;
   char *name;
   int flags = MU_STREAM_RDWR;
   
@@ -363,7 +390,7 @@ mh_open_folder (const char *folder, int create)
   if (create && mh_check_folder (name, 1))
     exit (0);
     
-  if (mailbox_create_default (&mbox, name))
+  if (mu_mailbox_create_default (&mbox, name))
     {
       mh_error (_("Cannot create mailbox %s: %s"),
 		name, strerror (errno));
@@ -373,7 +400,7 @@ mh_open_folder (const char *folder, int create)
   if (create)
     flags |= MU_STREAM_CREAT;
   
-  if (mailbox_open (mbox, flags))
+  if (mu_mailbox_open (mbox, flags))
     {
       mh_error (_("Cannot open mailbox %s: %s"), name, strerror (errno));
       exit (1);
@@ -402,44 +429,42 @@ mh_get_dir ()
 char *
 mh_expand_name (const char *base, const char *name, int is_folder)
 {
-  char *tmp = NULL;
   char *p = NULL;
-  char *namep;
+  char *namep = NULL;
   
-  tmp = mu_tilde_expansion (name, "/", NULL);
-  if (tmp[0] == '+')
-    namep = tmp + 1;
-  else if (strncmp (tmp, "../", 3) == 0 || strncmp (tmp, "./", 2) == 0)
+  namep = mu_tilde_expansion (name, "/", NULL);
+  if (namep[0] == '+')
+    memmove (namep, namep + 1, strlen (namep)); /* copy null byte as well */
+  else if (strncmp (namep, "../", 3) == 0 || strncmp (namep, "./", 2) == 0)
     {
       char *cwd = mu_getcwd ();
-      asprintf (&namep, "%s/%s", cwd, tmp);
+      char *tmp = NULL;
+      asprintf (&tmp, "%s/%s", cwd, namep);
       free (cwd);
-      free (tmp);
-      tmp = NULL;
+      free (namep);
+      namep = tmp;
     }
-  else
-    namep = tmp;
   
-  if (!base)
-    base = mu_path_folder_dir;
   if (is_folder)
     {
-      if (namep[0] == '/')
+      if (memcmp (namep, "mh:/", 4) == 0)
+	return namep;
+      else if (namep[0] == '/')
 	asprintf (&p, "mh:%s", namep);
       else
-	asprintf (&p, "mh:%s/%s", base, namep);
+	asprintf (&p, "mh:%s/%s", base ? base : mu_folder_directory (), namep);
     }
   else if (namep[0] != '/')
-    asprintf (&p, "%s/%s", base, namep);
+    asprintf (&p, "%s/%s", base ? base : mu_folder_directory (), namep);
   else
     return namep;
   
-  free (tmp);
+  free (namep);
   return p;
 }
 
 int
-mh_iterate (mailbox_t mbox, mh_msgset_t *msgset,
+mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
 	    mh_iterator_fp itr, void *data)
 {
   int rc;
@@ -447,11 +472,11 @@ mh_iterate (mailbox_t mbox, mh_msgset_t *msgset,
 
   for (i = 0; i < msgset->count; i++)
     {
-      message_t msg;
+      mu_message_t msg;
       size_t num;
 
       num = msgset->list[i];
-      if ((rc = mailbox_get_message (mbox, num, &msg)) != 0)
+      if ((rc = mu_mailbox_get_message (mbox, num, &msg)) != 0)
 	{
 	  mh_error (_("Cannot get message %d: %s"), num, mu_strerror (rc));
 	  return 1;
@@ -469,10 +494,10 @@ mh_spawnp (const char *prog, const char *file)
   int argc, i, rc, status;
   char **argv, **xargv;
 
-  if (argcv_get (prog, "", "#", &argc, &argv))
+  if (mu_argcv_get (prog, "", "#", &argc, &argv))
     {
       mh_error (_("Cannot split line %s"), prog);
-      argcv_free (argc, argv);
+      mu_argcv_free (argc, argv);
       return 1;
     }
 
@@ -480,7 +505,7 @@ mh_spawnp (const char *prog, const char *file)
   if (!xargv)
     {
       mh_err_memory (0);
-      argcv_free (argc, argv);
+      mu_argcv_free (argc, argv);
       return 1;
     }
 
@@ -492,7 +517,7 @@ mh_spawnp (const char *prog, const char *file)
   rc = mu_spawnvp (xargv[0], (const char**) xargv, &status);
 
   free (xargv);
-  argcv_free (argc, argv);
+  mu_argcv_free (argc, argv);
 
   return rc;
 }
@@ -503,8 +528,8 @@ mh_file_copy (const char *from, const char *to)
   char *buffer;
   size_t bufsize, rdsize;
   struct stat st;
-  stream_t in;
-  stream_t out;
+  mu_stream_t in;
+  mu_stream_t out;
   int rc;
   
   if (stat (from, &st))
@@ -520,8 +545,8 @@ mh_file_copy (const char *from, const char *to)
   if (!bufsize)
     mh_err_memory (1);
 
-  if ((rc = file_stream_create (&in, from, MU_STREAM_READ)) != 0
-      || (rc = stream_open (in)))
+  if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)) != 0
+      || (rc = mu_stream_open (in)))
     {
       mh_error (_("Cannot open input file `%s': %s"),
 		from, mu_strerror (rc));
@@ -529,22 +554,22 @@ mh_file_copy (const char *from, const char *to)
       return 1;
     }
 
-  if ((rc = file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
-      || (rc = stream_open (out)))
+  if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
+      || (rc = mu_stream_open (out)))
     {
       mh_error (_("Cannot open output file `%s': %s"),
 		to, mu_strerror (rc));
       free (buffer);
-      stream_close (in);
-      stream_destroy (&in, stream_get_owner (in));
+      mu_stream_close (in);
+      mu_stream_destroy (&in, mu_stream_get_owner (in));
       return 1;
     }
 
   while (st.st_size > 0
-	 && (rc = stream_sequential_read (in, buffer, bufsize, &rdsize)) == 0
+	 && (rc = mu_stream_sequential_read (in, buffer, bufsize, &rdsize)) == 0
 	 && rdsize > 0)
     {
-      if ((rc = stream_sequential_write (out, buffer, rdsize)) != 0)
+      if ((rc = mu_stream_sequential_write (out, buffer, rdsize)) != 0)
 	{
 	  mh_error (_("Write error on `%s': %s"),
 		    to, mu_strerror (rc));
@@ -555,20 +580,20 @@ mh_file_copy (const char *from, const char *to)
 
   free (buffer);
 
-  stream_close (in);
-  stream_close (out);
-  stream_destroy (&in, stream_get_owner (in));
-  stream_destroy (&out, stream_get_owner (out));
+  mu_stream_close (in);
+  mu_stream_close (out);
+  mu_stream_destroy (&in, mu_stream_get_owner (in));
+  mu_stream_destroy (&out, mu_stream_get_owner (out));
   
   return rc;
 }
 
-message_t
-mh_file_to_message (char *folder, char *file_name)
+mu_message_t
+mh_file_to_message (const char *folder, char *file_name)
 {
   struct stat st;
   int rc;
-  stream_t instream;
+  mu_stream_t instream;
   
   if (folder)
     file_name = mh_expand_name (folder, file_name, 0);
@@ -579,18 +604,18 @@ mh_file_to_message (char *folder, char *file_name)
       return NULL;
     }
   
-  if ((rc = file_stream_create (&instream, file_name, MU_STREAM_READ)))
+  if ((rc = mu_file_stream_create (&instream, file_name, MU_STREAM_READ)))
     {
       mh_error (_("Cannot create input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
       return NULL;
     }
   
-  if ((rc = stream_open (instream)))
+  if ((rc = mu_stream_open (instream)))
     {
       mh_error (_("Cannot open input stream (file %s): %s"),
 		file_name, mu_strerror (rc));
-      stream_destroy (&instream, stream_get_owner (instream));
+      mu_stream_destroy (&instream, mu_stream_get_owner (instream));
       return NULL;
     }
 
@@ -718,12 +743,12 @@ mh_install (char *name, int automode)
 }
         
 void
-mh_annotate (message_t msg, char *field, char *text, int date)
+mh_annotate (mu_message_t msg, char *field, char *text, int date)
 {
-  header_t hdr;
-  attribute_t attr;
+  mu_header_t hdr;
+  mu_attribute_t attr;
   
-  if (message_get_header (msg, &hdr))
+  if (mu_message_get_header (msg, &hdr))
     return;
 
   if (date)
@@ -735,20 +760,20 @@ mh_annotate (message_t msg, char *field, char *text, int date)
       tm = localtime (&t);
       strftime (datebuf, sizeof datebuf, "%a, %d %b %Y %H:%M:%S %Z", tm);
 
-      header_set_value (hdr, field, datebuf, 0);
+      mu_header_set_value (hdr, field, datebuf, 0);
     }
 
   if (text)
-    header_set_value (hdr, field, text, 0);
-  message_get_attribute (msg, &attr);
-  attribute_set_modified (attr);
+    mu_header_set_value (hdr, field, text, 0);
+  mu_message_get_attribute (msg, &attr);
+  mu_attribute_set_modified (attr);
 }
 
 char *
 mh_draft_name ()
 {
   char *draftfolder = mh_global_profile_get ("Draft-Folder",
-					     mu_path_folder_dir);
+					     mu_folder_directory ());
   return mh_expand_name (draftfolder, "draft", 0);
 }
 
@@ -784,7 +809,7 @@ void
 mh_set_reply_regex (const char *str)
 {
   char *err;
-  int rc = munre_set_regex (str, 0, &err);
+  int rc = mu_unre_set_regex (str, 0, &err);
   if (rc)
     mh_error ("reply_regex: %s%s%s", mu_strerror (rc),
 	      err ? ": " : "",
@@ -821,5 +846,137 @@ mh_decode_2047 (char *text, char **decoded_text)
   if (!charset)
     return 1;
   
-  return rfc2047_decode (charset, text, decoded_text);
+  return mu_rfc2047_decode (charset, text, decoded_text);
 }
+
+void
+mh_quote (const char *in, char **out)
+{
+  size_t len = strlen (in);
+  if (len && in[0] == '"' && in[len - 1] == '"')
+    {
+      const char *p;
+      char *q;
+      
+      for (p = in + 1; p < in + len - 1; p++)
+        if (*p == '\\' || *p == '"')
+	  len++;
+
+      *out = xmalloc (len + 1);
+      q = *out;
+      p = in;
+      *q++ = *p++;
+      while (p[1])
+	{
+	  if (*p == '\\' || *p == '"')
+	    *q++ = '\\';
+	  *q++ = *p++;
+	}
+      *q++ = *p++;
+      *q = 0;
+    }
+  else
+    *out = xstrdup (in);
+}
+
+void
+mh_expand_aliases (mu_message_t msg,
+		   mu_address_t *addr_to,
+		   mu_address_t *addr_cc,
+		   mu_address_t *addr_bcc)
+{
+  mu_header_t hdr;
+  size_t i, num;
+  char *buf;
+  
+  mu_message_get_header (msg, &hdr);
+  mu_header_get_field_count (hdr, &num);
+  for (i = 1; i <= num; i++)
+    {
+      if (mu_header_aget_field_name (hdr, i, &buf) == 0)
+	{
+	  if (strcasecmp (buf, MU_HEADER_TO) == 0
+	      || strcasecmp (buf, MU_HEADER_CC) == 0
+	      || strcasecmp (buf, MU_HEADER_BCC) == 0)
+	    {
+	      char *value;
+	      mu_address_t addr = NULL;
+	      int incl;
+	      
+	      mu_header_aget_field_value_unfold (hdr, i, &value);
+	      
+	      mh_alias_expand (value, &addr, &incl);
+	      free (value);
+	      if (strcasecmp (buf, MU_HEADER_TO) == 0)
+		mu_address_union (addr_to, addr);
+	      else if (strcasecmp (buf, MU_HEADER_CC) == 0)
+		mu_address_union (addr_cc ? addr_cc : addr_to, addr);
+	      else if (strcasecmp (buf, MU_HEADER_BCC) == 0)
+		mu_address_union (addr_bcc ? addr_bcc : addr_to, addr);
+	    }
+	  free (buf);
+	}
+    }
+}
+
+int
+mh_draft_message (const char *name, const char *msgspec, char **pname)
+{
+  mu_url_t url;
+  size_t uid;
+  int rc;
+  const char *urlstr;
+  mu_mailbox_t mbox;
+
+  mbox = mh_open_folder (name, 0);
+  if (!mbox)
+    return 1;
+  
+  mu_mailbox_get_url (mbox, &url);
+  urlstr = mu_url_to_string (url);
+
+  if (strcmp (msgspec, "new") == 0)
+    {
+      rc = mu_mailbox_uidnext (mbox, &uid);
+      if (rc)
+	mu_error (_("Cannot obtain sequence number for the new message"),
+		  mu_strerror (rc));
+    }
+  else
+    {
+      char *argv[2];
+      mh_msgset_t msgset;
+      
+      argv[0] = (char*) msgspec;
+      argv[1] = NULL;
+      rc = mh_msgset_parse (mbox, &msgset, 1, argv, "cur");
+      if (rc)
+	mu_error (_("Invalid message number: %s"), msgspec);
+      else if (msgset.count > 1)
+	mu_error (_("only one message at a time!"));
+      else
+	uid = msgset.list[0];
+
+      mh_msgset_free (&msgset);
+    }
+  
+  if (rc == 0)
+    {
+      const char *dir;
+      const char *msg;
+      size_t len;
+      
+      dir = urlstr + 3; /* FIXME */
+      
+      msg = mu_umaxtostr (0, uid);
+      len = strlen (dir) + 1 + strlen (msg) + 1;
+      *pname = xmalloc (len);
+      strcpy (*pname, dir);
+      strcat (*pname, "/");
+      strcat (*pname, msg);
+    }
+  mu_mailbox_close (mbox);
+  mu_mailbox_destroy (&mbox);
+  return rc;
+}
+
