@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 /* MH send command */
 
@@ -31,7 +32,6 @@ Options marked with `*' are not yet implemented.\n\
 Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("file [file...]");
 
-
 /* GNU options */
 static struct argp_option options[] = {
   {"alias",         ARG_ALIAS,         N_("FILE"), 0,
@@ -52,7 +52,7 @@ static struct argp_option options[] = {
    N_("* Reformat To: and Cc: addresses") },
   {"noformat",      ARG_NOFORMAT,      NULL, OPTION_HIDDEN, "" },
   {"forward",       ARG_FORWARD,       N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* In case of failure forward the draft along with the failure notice to the sender.") },
+   N_("* In case of failure forward the draft along with the failure notice to the sender") },
   {"noforward",     ARG_NOFORWARD,     NULL, OPTION_HIDDEN, "" },
   {"mime",          ARG_MIME,          N_("BOOL"), OPTION_ARG_OPTIONAL,
    N_("* Use MIME encapsulation") },
@@ -116,6 +116,8 @@ static int verbose;              /* Produce verbose diagnostics */
 static int watch;                /* Watch the delivery process */
 static unsigned width = 76;      /* Maximum width of header fields */
 
+#define DEFAULT_X_MAILER "MH (" PACKAGE_STRING ")"
+
 #define WATCH(c) do {\
   if (watch)\
     watch_printf c;\
@@ -147,7 +149,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
     case ARG_DRAFTMESSAGE:
       if (!draft_folder)
 	draft_folder = mh_global_profile_get ("Draft-Folder",
-					      mu_path_folder_dir);
+					      mu_folder_directory ());
       break;
       
     case ARG_FILTER:
@@ -155,7 +157,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       return 1;
       
     case ARG_FORMAT:
-      reformat_recipients = is_true(arg);
+      reformat_recipients = is_true (arg);
       break;
       
     case ARG_NOFORMAT:
@@ -163,7 +165,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_FORWARD:
-      forward_notice = is_true(arg);
+      forward_notice = is_true (arg);
       break;
       
     case ARG_NOFORWARD:
@@ -171,7 +173,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_MIME:
-      mime_encaps = is_true(arg);
+      mime_encaps = is_true (arg);
       break;
       
     case ARG_NOMIME:
@@ -179,7 +181,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_MSGID:
-      append_msgid = is_true(arg);
+      append_msgid = is_true (arg);
       break;
       
     case ARG_NOMSGID:
@@ -187,7 +189,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_PUSH:
-      background = is_true(arg);
+      background = is_true (arg);
       break;
       
     case ARG_NOPUSH:
@@ -196,7 +198,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       
     case ARG_SPLIT:
       split_message = 1;
-      split_interval = strtoul(arg, &p, 10);
+      split_interval = strtoul (arg, &p, 10);
       if (*p)
 	{
 	  argp_error (state, _("Invalid number"));
@@ -205,7 +207,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_VERBOSE:
-      verbose = is_true(arg);
+      verbose = is_true (arg);
       break;
       
     case ARG_NOVERBOSE:
@@ -213,7 +215,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_WATCH:
-      watch = is_true(arg);
+      watch = is_true (arg);
       break;
       
     case ARG_NOWATCH:
@@ -221,7 +223,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_WIDTH:
-      width = strtoul(arg, &p, 10);
+      width = strtoul (arg, &p, 10);
       if (*p)
 	{
 	  argp_error (state, _("Invalid number"));
@@ -250,84 +252,113 @@ watch_printf (const char *fmt, ...)
   va_end (ap);
 }
 
-static list_t mesg_list;
+static mu_list_t mesg_list;
 static mh_context_t *mts_profile;
 
 int
 check_file (char *name)
 {
-  message_t msg;
+  mu_message_t msg;
 
   msg = mh_file_to_message (draft_folder, name);
   if (!msg)
     return 1;
-  if (!mesg_list && list_create (&mesg_list))
+  if (!mesg_list && mu_list_create (&mesg_list))
     {
-      mh_error (_("can't create message list"));
+      mh_error (_("Cannot create message list"));
       return 1;
     }
   
-  return list_append (mesg_list, msg);
+  return mu_list_append (mesg_list, msg);
 }
 
 void
 read_mts_profile ()
 {
   char *p;
-
+  char *hostname = NULL;
+  int rc;
+  mh_context_t *local_profile;
+  
   p = mh_expand_name (MHLIBDIR, "mtstailor", 0);
-  if (!p)
-    {
-      char *home = mu_get_homedir ();
-      if (!home)
-	abort (); /* shouldn't happen */
-      asprintf (&p, "%s/%s", home, ".mtstailor");
-      free (home);
-    }
   mts_profile = mh_context_create (p, 1);
   mh_context_read (mts_profile);
+
+  p = mu_tilde_expansion ("~/.mtstailor", "/", NULL);
+  local_profile = mh_context_create (p, 1);
+  if (mh_context_read (local_profile) == 0)
+    mh_context_merge (mts_profile, local_profile);
+  mh_context_destroy (&local_profile);
+
+  if ((p = mh_context_get_value (mts_profile, "localname", NULL)))
+    {
+      hostname = p;
+      mu_set_user_email_domain (p);
+    }
+  else if ((rc = mu_get_host_name (&hostname)))
+    mu_error (_("Cannot get system host name: %s"), mu_strerror (rc));
+  
+  if ((p = mh_context_get_value (mts_profile, "localdomain", NULL)))
+    {
+      char *newdomain;
+
+      if (!hostname)
+	exit (1);
+      
+      newdomain = xmalloc (strlen (hostname) + 1 + strlen (p) + 1);
+      strcpy (newdomain, hostname);
+      strcat (newdomain, ".");
+      strcat (newdomain, p);
+      rc = mu_set_user_email_domain (newdomain);
+      free (newdomain);
+      if (rc)
+	{
+	  mu_error (_("Cannot set user mail domain: %s"), mu_strerror (rc));
+	  exit (1);
+	}
+    }
 }
 
 
-mailer_t
+mu_mailer_t
 open_mailer ()
 {
   char *url = mh_context_get_value (mts_profile,
 				    "url",
 				    "sendmail:/usr/sbin/sendmail");
-  mailer_t mailer;
+  mu_mailer_t mailer;
   int status;
     
-  WATCH(("creating mailer %s", url));
-  status = mailer_create (&mailer, url);
+  WATCH ((_("Creating mailer %s"), url));
+  status = mu_mailer_create (&mailer, url);
   if (status)
     {
-      mh_error(_("cannot create mailer \"%s\""), url);
+      mh_error (_("Cannot create mailer `%s'"), url);
       return NULL;
     }
 
   if (verbose)
     {
       mu_debug_t debug = NULL;
-      mailer_get_debug (mailer, &debug);
+      mu_mailer_get_debug (mailer, &debug);
       mu_debug_set_level (debug, MU_DEBUG_TRACE | MU_DEBUG_PROT);
     }
 
-  WATCH(("opening mailer %s", url));
-  status = mailer_open (mailer, MU_STREAM_RDWR);
+  WATCH ((_("Opening mailer %s"), url));
+  status = mu_mailer_open (mailer, MU_STREAM_RDWR);
   if (status)
     {
-      mh_error(_("cannot open mailer \"%s\""), url);
+      mh_error (_("Cannot open mailer `%s'"), url);
       return NULL;
     }
   return mailer;
 }
 
 static void
-create_message_id (header_t hdr)
+create_message_id (mu_header_t hdr)
 {
   char *p = mh_create_message_id (0);
-  header_set_value (hdr, MU_HEADER_MESSAGE_ID, p, 1);
+  mu_header_set_value (hdr, MU_HEADER_MESSAGE_ID, p, 1);
   free (p);
 }
 
@@ -350,84 +381,60 @@ get_sender_personal ()
 }
 
 static void
-set_address_header (header_t hdr, char *name, address_t addr)
+set_address_header (mu_header_t hdr, char *name, mu_address_t addr)
 {
-  size_t s = address_format_string (addr, NULL, 0);
+  size_t s = mu_address_format_string (addr, NULL, 0);
   char *value = xmalloc (s + 1);
-  address_format_string (addr, value, s);
-  header_set_value (hdr, name, value, 1);
+  mu_address_format_string (addr, value, s);
+  mu_header_set_value (hdr, name, value, 1);
   free (value);
 }
 
 void
-expand_aliases (message_t msg)
+expand_aliases (mu_message_t msg)
 {
-  header_t hdr;
-  size_t i, num;
-  char buf[16];
-  address_t addr_to = NULL,
-            addr_cc = NULL,
-            addr_bcc = NULL;
-  
-  message_get_header (msg, &hdr);
-  header_get_field_count (hdr, &num);
-  for (i = 1; i <= num; i++)
-    {
-      header_get_field_name (hdr, i, buf, sizeof buf, NULL);
-      if (strcasecmp (buf, MU_HEADER_TO) == 0
-	  || strcasecmp (buf, MU_HEADER_CC) == 0
-	  || strcasecmp (buf, MU_HEADER_BCC) == 0)
-	{
-	  char *value;
-	  address_t addr = NULL;
-	  int incl;
-	  
-	  header_aget_field_value_unfold (hdr, i, &value);
-      
-	  mh_alias_expand (value, &addr, &incl);
-	  free (value);
-	  if (strcasecmp (buf, MU_HEADER_TO) == 0)
-	    address_union (&addr_to, addr);
-	  else if (strcasecmp (buf, MU_HEADER_CC) == 0)
-	    address_union (&addr_cc, addr);
-	  else if (strcasecmp (buf, MU_HEADER_BCC) == 0)
-	    address_union (&addr_bcc, addr);
-	}
-    }
+  mu_header_t hdr;
+  mu_address_t addr_to = NULL,
+               addr_cc = NULL,
+               addr_bcc = NULL;
 
+  mh_expand_aliases (msg, &addr_to, &addr_cc, &addr_bcc);
+
+  mu_message_get_header (msg, &hdr);
   if (addr_to)
     {
       set_address_header (hdr, MU_HEADER_TO, addr_to);
-      address_destroy (&addr_to);
+      mu_address_destroy (&addr_to);
     }
 
   if (addr_cc)
     {
       set_address_header (hdr, MU_HEADER_CC, addr_cc);
-      address_destroy (&addr_cc);
+      mu_address_destroy (&addr_cc);
     }
 
   if (addr_bcc)
     {
       set_address_header (hdr, MU_HEADER_BCC, addr_bcc);
-      address_destroy (&addr_bcc);
+      mu_address_destroy (&addr_bcc);
     }
 }
 
 void
-fix_fcc (message_t msg)
+fix_fcc (mu_message_t msg)
 {
-  header_t hdr;
+  mu_header_t hdr;
   char *val;
   
-  message_get_header (msg, &hdr);
-  if (header_aget_value (hdr, MU_HEADER_FCC, &val) == 0
+  mu_message_get_header (msg, &hdr);
+  if (mu_header_aget_value (hdr, MU_HEADER_FCC, &val) == 0
       && strchr ("+%~/=", val[0]) == NULL)
     {
       val = realloc (val, strlen (val) + 2);
       memmove (val + 1, val, strlen (val) + 1);
       val[0] = '+';
-      header_set_value (hdr, MU_HEADER_FCC, val, 1);
+      mu_header_set_value (hdr, MU_HEADER_FCC, val, 1);
+      WATCH ((_("Fixed Fcc: %s"), val));
       free (val);
     }  
 }
@@ -435,24 +442,24 @@ fix_fcc (message_t msg)
 int
 _action_send (void *item, void *data)
 {
-  message_t msg = item;
+  mu_message_t msg = item;
   int rc;
-  mailer_t mailer;
-  header_t hdr;
+  mu_mailer_t mailer;
+  mu_header_t hdr;
   size_t n;
 
-  WATCH(("Getting message"));
+  WATCH ((_("Getting message")));
 
-  if (message_get_header (msg, &hdr) == 0)
+  if (mu_message_get_header (msg, &hdr) == 0)
     {
       char date[80];
       time_t t = time (NULL);
       struct tm *tm = localtime (&t);
       
       strftime (date, sizeof date, "%a, %d %b %Y %H:%M:%S %Z", tm);
-      header_set_value (hdr, MU_HEADER_DATE, date, 1);
+      mu_header_set_value (hdr, MU_HEADER_DATE, date, 1);
 
-      if (header_get_value (hdr, MU_HEADER_FROM, NULL, 0, &n))
+      if (mu_header_get_value (hdr, MU_HEADER_FROM, NULL, 0, &n))
 	{
 	  char *from;
 	  char *email = mu_get_user_email (NULL);
@@ -465,15 +472,26 @@ _action_send (void *item, void *data)
 	  else
 	    from = email;
 
-	  header_set_value (hdr, MU_HEADER_FROM, from, 1);
+	  mu_header_set_value (hdr, MU_HEADER_FROM, from, 1);
 	  free (from);
 	}
 	  
       if (append_msgid
-	  && header_get_value (hdr, MU_HEADER_MESSAGE_ID, NULL, 0, &n))
+	  && mu_header_get_value (hdr, MU_HEADER_MESSAGE_ID, NULL, 0, &n))
 	create_message_id (hdr);
-    }
 
+      if (mu_header_get_value (hdr, MU_HEADER_X_MAILER, NULL, 0, &n))
+	{
+	  char *p = mh_context_get_value (mts_profile, "x-mailer", "yes");
+
+	  if (!strcmp (p, "yes"))
+	    mu_header_set_value (hdr, MU_HEADER_X_MAILER,
+				 DEFAULT_X_MAILER, 0);
+	  else if (strcmp (p, "no"))
+	    mu_header_set_value (hdr, MU_HEADER_X_MAILER, p, 0);
+	}
+    }
+  
   expand_aliases (msg);
   fix_fcc (msg);
   
@@ -481,17 +499,17 @@ _action_send (void *item, void *data)
   if (!mailer)
     return 1;
 
-  WATCH(("Sending message"));
-  rc = mailer_send_message (mailer, msg, NULL, NULL);
+  WATCH ((_("Sending message")));
+  rc = mu_mailer_send_message (mailer, msg, NULL, NULL);
   if (rc)
     {
-      mh_error(_("cannot send message: %s"), mu_strerror (rc));
+      mh_error(_("Cannot send message: %s"), mu_strerror (rc));
       return 1;
     }
 
-  WATCH(("Destroying the mailer"));
-  mailer_close (mailer);
-  mailer_destroy (&mailer);
+  WATCH ((_("Destroying the mailer")));
+  mu_mailer_close (mailer);
+  mu_mailer_destroy (&mailer);
   
   return 0;
 }
@@ -513,17 +531,18 @@ send (int argc, char **argv)
   
   if (background && daemon (0, 0) < 0)
     {
-      mh_error(_("cannot switch to background: %s"), mu_strerror (errno));
+      mh_error(_("Cannot switch to background: %s"), mu_strerror (errno));
       return 1;
     }
 
   /* Prepend url specifier to the folder dir. We won't need this
      when the default format becomes configurable */
-  asprintf (&p, "mh:%s", mu_path_folder_dir);
-  mu_path_folder_dir = p;
+  asprintf (&p, "mh:%s", mu_folder_directory ());
+  mu_set_folder_directory (p);
+  free (p);
   
   /* Finally, do the work */
-  rc = list_do (mesg_list, _action_send, NULL);
+  rc = mu_list_do (mesg_list, _action_send, NULL);
   return rc;
 }
 	  
@@ -562,5 +581,5 @@ main (int argc, char **argv)
       argc = 1;
     }
 
-  return send(argc, argv);  
+  return send (argc, argv);  
 }
