@@ -13,7 +13,8 @@
 
    You should have received a copy of the GNU General Public License
    along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA  */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+   MA 02110-1301 USA */
 
 #include "pop3d.h"
 #include <mailutils/argcv.h>
@@ -39,13 +40,13 @@ struct action_data {
 };
 
 void check_action(int action);
-int action_create __P((struct action_data *ap));
-int action_add __P((struct action_data *ap));
-int action_delete __P((struct action_data *ap));
-int action_list __P((struct action_data *ap));
-int action_chpass __P((struct action_data *ap));
+int action_create (struct action_data *ap);
+int action_add (struct action_data *ap);
+int action_delete (struct action_data *ap);
+int action_list (struct action_data *ap);
+int action_chpass (struct action_data *ap);
 
-int (*ftab[]) __P((struct action_data *)) = {
+int (*ftab[]) (struct action_data *) = {
   action_create,
   action_add,
   action_delete,
@@ -55,12 +56,11 @@ int (*ftab[]) __P((struct action_data *)) = {
 
 const char *program_version = "popauth (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU popauth -- manage pop3 authentication database");
-static error_t popauth_parse_opt  __P((int key, char *arg,
-				       struct argp_state *astate));
+static error_t popauth_parse_opt  (int key, char *arg,
+				   struct argp_state *astate);
 
-void popauth_version __P((FILE *stream, struct argp_state *state));
-void (*argp_program_version_hook) __P((FILE *stream,
-				       struct argp_state *state)) =
+void popauth_version (FILE *stream, struct argp_state *state);
+void (*argp_program_version_hook) (FILE *stream, struct argp_state *state) =
                                    popauth_version;
 
 static struct argp_option options[] = 
@@ -382,22 +382,22 @@ action_create (struct action_data *ap)
 	buf[--len] = 0;
       
       line++;
-      if (argcv_get (buf, ":", NULL, &argc, &argv))
+      if (mu_argcv_get (buf, ":", NULL, &argc, &argv))
 	{
-	  argcv_free (argc, argv);
+	  mu_argcv_free (argc, argv);
 	  continue;
 	}
 
       if (argc == 0 || argv[0][0] == '#')
 	{
-	  argcv_free (argc, argv);
+	  mu_argcv_free (argc, argv);
 	  continue;
 	}
       
       if (argc != 3 || argv[1][0] != ':' || argv[1][1] != 0)
 	{
 	  mu_error (_("%s:%d: malformed line"), ap->input_name, line);
-	  argcv_free (argc, argv);
+	  mu_argcv_free (argc, argv);
 	  continue;
 	}
 
@@ -411,7 +411,7 @@ action_create (struct action_data *ap)
       if (mu_dbm_insert (db, key, contents, 1))
 	mu_error (_("%s:%d: cannot store datum"), ap->input_name, line);
 
-      argcv_free (argc, argv);
+      mu_argcv_free (argc, argv);
     }
   mu_dbm_close (db);
   fclose (fp);
